;+
; NAME:     CANOPUS_TIME
; PURPOSE:  Returns Canopus Time, given the calendar date. Code is all IDL.
;           CANOPUS time is defined as seconds since 00:00:00 January 1, 1986.
; CALLING SEQUENCE:
;           CANOPUS_TIME,year,month,date,hour,minute,second,can_time,ct_error
; INPUTS:   YEAR, MONTH, DATE, HOUR, MINUTE, SECOND, CT_ERROR. 
;           If called with MONTH = 0 then DATE is assumed to be the 
;           day-of-year (January 1 = 1). 
;           Input ranges:
;              1919 <= YEAR   <= 2053
;                 0 <= MONTH  <= 12
;                 1 <= DAY    <= 31, or <= 367 if MONTH = 0
;                 0 <= HOUR   <= 24
;                 0 <= MINUTE <= 59
;                 0 <= SECOND <= 59
;           All inputs are checked - if valid range is exceeded then CAN_TIME is
;           set to -(2^31-1) = -2147483647 and CT_ERROR is returned as true.
; OUTPUTS:  CAN_TIME (Canopus Time).
;           CT_ERROR is returned as true if an error, otherwise false.
; PROCEDURE:
;           Calls user library routines CALDAT and JULDAT for date conversions.
;           Does not call a FORTRAN image.
; MODIFICATION HISTORY:
;           14 Sep 1993, GRANT, UWO: Written by modifying CANOPUS_TIME.PRO
;              (Written: F.Creutzberg 5-APR-1989, Last Modified: 20-NOV-1992
;              03:04:06.41).
;-

pro canopus_time, year,month,date,hour,minute,second,can_time,ct_error

on_error, 2
if n_params(0) ne 8 then begin
   print,'You MUST provide 8 parameters: Yr, Mo, Da, Hr, Mn, Sc, C_T, Err'
   return
endif

;    Test inputs. Return bad value if out of range.

can_time = -2147483647
ct_error = 1

if ((year  le 1918) or (year  ge 2054)) then goto, error_exit
if ((month lt 0)    or (month gt 12  )) then goto, error_exit
if (month eq 0) then begin
   if ((date lt 1) or (date gt 367)) then goto, error_exit
   endif $
else begin
   if ((date lt 1) or (date gt 31))  then goto, error_exit
   endelse
if ((hour   lt 0) or (hour   gt 24)) then goto, error_exit
if ((minute lt 0) or (minute gt 59)) then goto, error_exit
if ((second lt 0) or (second gt 59)) then goto, error_exit

yr = long(year)
mo = long(month)
da = long(date)
hr = long(hour)
mn = long(minute)
sc = long(second)

jd0 = julday(1,1,1986)

if month eq 0 then $
   jd = julday(1L,1L,yr) + da - 1L $
else $
   jd = julday(mo,da,yr)

can_time = (jd-jd0)*86400L + hr*3600L + mn*60L + sc

ct_error = 0
return


error_exit:

ct_error = 1
print,'CANOPUS Time Error'
return
end
