;+
; FILENAMENAME:
;       CADI_READ_DIR
;
;
;+
; NAME:
;       CADI_READ_FT
; PURPOSE:
;       Read a CADI FHT or FFI file into a structured variable.
; CALLING SEQUENCE:
;       Result = CADI_READ_FT( FILENAME )
; INPUTS:
;       FILENAME = scalar string. Fully qualified name of data file.
; KEYWORD PARAMETERS:
;       ONLYHEADER: If present and nonzero, only information from the file
;          header is returned. The structure returned is that normally
;          returned as Result.HEADER, but some fields such as those relating
;          to the times in the file are not present.
;             This option saves time and storage if only the file header needs
;          to be read.
;       ERROR: Scalar, returned as nonzero if an error occurs.
;       INFORM: If present and nonzero, progress messages will be printed.
;       ONLYX: If present and nonzero, only the X field of CADI.DATA is
;          returned, not the IQ field. If this option is set then file reading
;          is much faster and memory use is much less, and so this option is
;          recommended if the IQ data is not required, e.g. for h't plots.
;       TFH: For this to be set, ONLYX must also be set. If TFH is present and
;          nonzero, only the T, F and H fields of CADI.DATA.X are returned,
;          and not the D field, and duplicate T-F-H combinations are
;          eliminated. In other words, all Doppler and IQ information in the
;          files is ignored. This suits the usual h't plot. Use of /TFH results
;          in an even smaller storage requirement than when /ONLYX is used
;          alone.
; OUTPUTS:
;       Result = Variable of CADI structure, with IQ data if ONLYX is not set;
;          or just the CADI.HEADER structure if ONLYHEADER is set.
; RESTRICTIONS:
;       The returned structure was designed for MDn files, so there might be a
;          few anomalies in fitting FHT/FFI data into it.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 16 Sep 1994: Written by adapting CADI_READ_MD,
;          CADI_FT_OPEN/NEXT/CLOSE.
;       GRANT, 3 Feb 1995: Print message when I/O error detected.
;		MacDougall, 4 Dec 1998: pps set to 40 for EUR (error in header)
;-

function cadi_read_ft, filename, onlyheader=onlyheader, $
   error=error, inform=inform, onlyx=onlyx, tfh=tfh

on_error,0
error = !false
;on_ioerror, handle_ioerror

if keyword_set(tfh) then $
   if not keyword_set(onlyx) then begin
      message, /inform, 'If TFH is set then ONLYX must also be set'
      goto, quit
      endif

max_ntimes = 370L       ; One hour of 10-second samples
max_ndopbins = 100000L
nantennas = 4L          ; Not in file
dheight = 3.0           ; Not in file
minheight = 0           ; Not in file

; Read header
; -----------

header = {site:                 strof(' ',3), $
          ascii_datetime:       strof(' ',22), $
          filetype:             ' ', $
          nfreqs:               0, $
          ndops:                0B, $
          nheights:             0, $
          pps:                  0B, $
          npulses_avgd:         0B, $
          base_noise_thr:       0, $
          min_dop_forsave:      0B, $
          dtime:                0B}

openr, lun, filename, /get_lun, bufsiz=0 ;, /block
if eof(lun) then begin
   message, /inform, 'File ' + filename + ' is empty'
   goto, close_and_quit
   endif
readu, lun, header

; Process header
; --------------

if header.site eq 'R_B' then $  ; Mistake in R_B CADI: 20 Hz in file header.
   header.pps = 40B
if header.site eq 'EUR' then $  ; Mistake in some EUR CADI: 1 Hz in file header.
   header.pps = 40B
if header.site eq 'RAN' then $  ; Mistake in some RAN CADI: 1 Hz in file header.
   header.pps = 40B

; Decode ASCII_DATETIME for conversion to CANOPUS time

month='' & day=0 & hour=0 & min=0 & sec=0 & year=0
reads, header.ascii_datetime, month, day, hour, min, sec, year, $
   format='(x,a3,x,i2,3(x,i2),x,i4)'
time_header = cantime(year, month_number(month), day,hour,min,sec)
time_hour = 3600L * (time_header/3600L)

; Read frequencies
; ----------------

freqs = fltarr(header.nfreqs)
if header.filetype eq 'T' then begin

   ; Read frequencies from FHT file

   freq_bytes = bytarr(4)
   for freqx=0,header.nfreqs-1 do begin
      readu, lun, freq_bytes
       freqs(freqx)= FLOAT(freq_bytes,0)  ;added oct 3 1995, Bjj for PC only
      ;freqs(freqx) = ieee_to_float(freq_bytes)
      endfor
   endif ; header.filetype eq 'T'

; Form HEADER_OUT now in case it's the only thing wanted.
; But don't set fields NTIMES and TIMES, since they are unknown before the
; whole file is read.
; Set NTIMES to zero for now. Append TIMES field later.

header_out = { $
         nantennas:     long(nantennas), $
         ndops:         long(header.ndops), $
         nheights:      long(header.nheights), $
         ntimes:        0L, $                   ; Unknown before whole file read
         nfreqs:        long(header.nfreqs), $
         site:          header.site, $
         time:          time_header, $
         filetype:      header.filetype, $
         minheight:     fix(minheight), $
         maxheight:     fix(minheight+dheight*(header.nheights-1)), $
         pps:           header.pps, $
         npulses_avgd:  header.npulses_avgd, $
         base_thr100:   header.base_noise_thr, $ ; %% Changes from FT to MD
         noise_thr100:  0, $                     ; %% Changes from FT to MD
         min_dop_forsave:header.min_dop_forsave, $
         dtime:         long(header.dtime), $
         gain_control:  0B, $                   ; Not in FHT
         sig_process:   0B, $                   ; Not in FHT
         dops:          ( findgen(header.ndops) - (header.ndops/2) ) / $
                           (float(header.ndops) * $
                            header.npulses_avgd / float(header.pps)), $
         heights:       dheight*indgen(header.nheights), $
         freqs:         freqs}

if keyword_set(onlyheader) then begin
   close, lun
   free_lun, lun
   return, header_out
   endif

; Create variables to read and store data from file
; -------------------------------------------------

; Create DOPBINS, a vector of structures, to hold data for the
; time-frequency-height-Doppler bins.

; If /ONLYX, each element has only a field X.
; Otherwise, each element has fields X and IQ.

; X structure holds indices to time, freq, height, doppler
dopbin = {cadi_dopbrf_stu, x: {cadi_dop_x_stu, t:0, f:0, h:0, d:0B} }
if not keyword_set(onlyx) then $
   dopbin = $
      create_struct(name='cadi_dop_stu', dopbin, 'IQ', bytarr(2,nantennas) )
dopbins = replicate(dopbin, max_ndopbins)

; Create FREBINS, a vector of structures, to hold data describing the
; time-frequency bins.

if header.filetype eq 'F' then $
   max_nfrebins = header.nfreqs $       ; One time in file; probably ionogram
else $
   ; Many times in file
   max_nfrebins = min([ max_ntimes * header.nfreqs, max_ndopbins ])
max_nfrebins = min([ max_ntimes * header.nfreqs, max_ndopbins ]) ;%%

frebins = replicate({cadi_frebin_stu, $
   x:  {cadi_fre_x_stu, t:0B, f:0B}, $
   gain_flag:     0B, $
   noise_flag:    0B, $
   noise_power10: 0}, $
      max_nfrebins)

times = lonarr(max_ntimes)

flag = 0B
hflag = 0
time_min = 0B
time_sec = 0B
noise_flag = 0B
noise_power = 0
ndops_oneh = 0B
dop_flag = 0B
freqtime_bytes = bytarr(4)
iq_bytes = bytarr(2,nantennas)

; Initialize indices etc.

old_time = -1L  ; Initialize to bad time
dopbinx = -1L
frebinx = -1L
if header.filetype eq 'T' then $
   timex = -1L $
   ; Don't need to initialize FREQX for FHT
else begin
   ; For ionogram, the one time is in header
   timex = 0L
   times = [time_header]
   freqx = -1L
   endelse
if keyword_set(inform) then print, 'Processing time index ...'

; Read first flag
; ---------------

if not eof(lun) then readu, lun, flag

; Loop for each time-frequency bin
; --------------------------------

; There is only one time for FILETYPE eq 'F' (ionogram).
;stop
next_ft:

   ; Read data for this freq or time, except freq/time FLAG which was read already

   readu, lun, freqtime_bytes, noise_flag, noise_power  ; 7 bytes

   ; Determine FREQX and TIMEX for this ft, according to FILETYPE

   if header.filetype eq 'F' then begin
      freqx = freqx+1
      ;header_out.freqs(freqx) = ieee_to_float(freqtime_bytes)
      header_out.freqs(freqx)= FLOAT( freqtime_bytes, 0 )
      endif $
   else begin
      freqx = flag - 'F0'XB
      time = time_hour + $ ; Form time from min and sec fields
         60L * fix(freqtime_bytes, 0) + fix(freqtime_bytes, 2)
      if time ne old_time then begin
         timex = timex + 1
         times(timex) = time
         old_time = time
         if keyword_set(inform) then print, timex, format='($,i5)'
         endif
      endelse

   ; Store data for this time-frequency bin

   frebinx = frebinx + 1

   frebins(frebinx).x = {cadi_fre_x_stu, timex, freqx}
   frebins(frebinx).noise_flag = noise_flag
   frebins(frebinx).noise_power10 = noise_power ;%% Not same in FHT and MD

   flag = 0B
   next_height: ; End of this freq/time is indicated by EOF or FLAG GE E0
      if eof(lun) then goto, ft_done

      readu, lun, flag

      if flag ge 'E0'XB then goto, ft_done

      ; Start a new height

      readu, lun, ndops_oneh    ; TNUM in data collection program

      if keyword_set(tfh) then begin

         ; Read past spectra for this height. FLAG is height index.

         dopbinx = dopbinx + 1
         if dopbinx ge max_ndopbins then begin
            message, /inform, 'Too many doppler bins in file'
            goto, close_and_quit
            endif
         dopbins(dopbinx).x = {cadi_dop_x_stu, timex, freqx, flag, 0}
         for dopx=0,ndops_oneh-1 do begin

            readu, lun, dop_flag

          ;  IF (frebinx EQ 9) THEN stop
            readu, lun, iq_bytes
         ENDFOR

         endif $
      else $

;print,ndops_oneh,dop_flag,iq_bytes

         ; Read spectra for this height. FLAG is height index.

         for dopx=0,ndops_oneh-1 do begin

            readu, lun, dop_flag, iq_bytes
        ;   print,dop_flag,iq_bytes
            dopbinx = dopbinx + 1

            if dopbinx ge max_ndopbins then begin
               message, /inform, 'Too many doppler bins in file'
               goto, close_and_quit
               endif
            dopbins(dopbinx).x = {cadi_dop_x_stu, timex, freqx, flag, dop_flag}
            if not keyword_set(onlyx) then $
               dopbins(dopbinx).iq = iq_bytes
            endfor ; dopx
        ;PRINT,noise_flag,noise_power,flag,ndops_oneh,dop_flag,iq_bytes

      goto, next_height
   ft_done:
   if not eof(lun) then goto, next_ft

; Have reached end of file

if keyword_set(inform) then print, ''
close, lun
free_lun, lun

ntimes = timex + 1
ndopbins = dopbinx + 1
nfrebins = frebinx + 1

if ntimes gt 0 then $
   times = times(0:ntimes-1) $
else $
   times = -1L

if ndopbins gt 0 then $
   dopbins = dopbins(0:ndopbins-1) $            ; Trim unused elements
else $
   dopbins = -1L

if nfrebins gt 0 then $
   frebins = frebins(0:nfrebins-1) $
else $
   frebins = -1L

; Decode Doppler bin flags
; ------------------------

if (ndopbins gt 0) and (not keyword_set(tfh)) then begin

   ; Very occasionally get a bad DOP_FLAG
   if howmany((dopbins.x.d lt 0) or $
              (dopbins.x.d ge header.ndops)) gt 0 then begin
      message, /inform, 'Bad DOP_FLAG'
      goto, quit
      endif

   half_ndops = byte( long(header.ndops) / 2 )
   w_lo = where(dopbins.x.d lt half_ndops, lo_count)
   w_hi = where(dopbins.x.d ge half_ndops, hi_count)
   if lo_count gt 0 then dopbins(w_lo).x.d = dopbins(w_lo).x.d + half_ndops
   if hi_count gt 0 then dopbins(w_hi).x.d = dopbins(w_hi).x.d - half_ndops
   endif

; Complete the output variables and return
; ----------------------------------------

header_out = create_struct(header_out, 'TIMES', times)  ; Append TIMES
header_out.ntimes = long(ntimes)

form_result:

header_exists = (datatype(header_out) eq 'STU')
if header_exists then $
   header_out = create_struct(header_out, 'DIR', '', 'FILE_EXT', '')

if datatype(frebins) eq 'STU' then $
   nfrebins = n_elements(frebins) $
else $
   nfrebins = 0L

if datatype(dopbins) eq 'STU' then $
   ndata = n_elements(dopbins) $
else $
   ndata = 0L

if keyword_set(onlyx) then $
   data_type = 'X' $
else $
   data_type = 'IQ'

return, create_struct( $
   'HEADER_EXISTS', header_exists, $
   'NFREBINS', nfrebins, $
   'NDATA', ndata, $
   'DATATYPE', data_type, $
   'HEADER', header_out, $
   'FREBINS', frebins, $
   'DATA', dopbins)

; Error handling
; --------------

handle_ioerror:
   message, /inform, 'I/O error reading file ' +!err_string
   stop
close_and_quit:
   close, lun
   free_lun, lun
quit:
   error = !true
   dopbins = -1L
   header_out = -1L
   frebins = -1L
   goto, form_result

end
;+
; NAME:
;       CADI_READ_MD
; PURPOSE:
;       Read a CADI MDn file into a structured variable.
; CALLING SEQUENCE:
;       Result = CADI_READ_MD( FILENAME )
; INPUTS:
;       FILENAME = scalar string. Fully qualified name of data file.
; KEYWORD PARAMETERS:
;       ERROR: Scalar, returned as nonzero if an error occurs.
;       INFORM: If present and nonzero, progress messages will be printed.
;       ONLYX: If present and nonzero, only the X field of CADI.DATA is
;          returned, not the IQ field. If this option is set then file reading
;          is much faster and memory use is much less, and so this option is
;          recommended if the IQ data is not required, e.g. for h't plots.
;       TFH: For this to be set, ONLYX must also be set. If TFH is present and
;          nonzero, only the T, F and H fields of CADI.DATA.X are returned,
;          and not the D field, and duplicate T-F-H combinations are
;          eliminated. In other words, all Doppler and IQ information in the
;          files is ignored. This suits the usual h't plot. Use of /TFH results
;          in an even smaller storage requirement than when /ONLYX is used
;          alone.
;       ONLYHEADER: If present and nonzero, only information from the file
;          header is returned. The structure returned is that normally
;          returned as Result.HEADER, but some fields such as those relating
;          to the times in the file are not present.
;             This option saves time and storage if only the file header needs
;          to be read.
; OUTPUTS:
;       Result = Variable of CADI structure, with IQ data if ONLYX is not set;
;          or just the CADI.HEADER structure if ONLYHEADER is set.
; RESTRICTIONS:
;       Fixes mistake in R_B CADI: 20 Hz is in file header for PPS, but this
;          routine changes that to 40 Hz.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 21 Mar 1994: Written.
;       GRANT, 3 Aug 1994: Added ONLYHEADER keyword.
;       GRANT, 30 Jan 1995: Combined appending of TIMES, DIR and FILE_EXT
;          fields to header, to avoid a TIMES field of dimension 1 being
;          converted to a scalar.
;       GRANT, 3 Feb 1995: Print message if I/O error detected.
;		JWM, 19 Aug 1997: extended height range using tnum flag implemented
;          and to have extended height range changed h from byte to int
;		JWM, 26 Sept 1997: extended height range mods, added htlimit keyword
;		JWM, 27 Oct 1997: added pulse rate correction for Fortaleza
;		JWM,  4 Dec 1998: added pulse correction for Eureka
;       JWM,  10 May 2001; added read of noofantennas from header
;-

function cadi_read_md, filename, $
   error=error, inform=inform, onlyx=onlyx, tfh=tfh, onlyheader=onlyheader, htlimitmin=ht_limitmin, htlimitmax = ht_limitmax
;stop
on_error, 2
error = !false
on_ioerror, handle_ioerror
;stop
if keyword_set(tfh) then $
   if not keyword_set(onlyx) then begin
      message, /inform, 'If TFH is set then ONLYX must also be set'
      goto, quit
      endif

max_ntimes = 256L       ;%% 1000L
max_ndopbins = 100000L
nantennas = 4L          ; Not in file
dheight = 3.0           ; Not in file

; Read header
; -----------

header = {site:                 strof(' ',3), $
          ascii_datetime:       strof(' ',22), $
          filetype:             ' ', $
          nfreqs:               0, $
          ndops:                0B, $
          minheight:            0, $
          maxheight:            0, $
          pps:                  0B, $
          npulses_avgd:         0B, $
          base_thr100:          0, $
          noise_thr100:         0, $
          min_dop_forsave:      0B, $
          dtime:                0, $
          gain_control:         0B, $
          sig_process:          0B, $
          noofreceivers:        0B,$

          spares:               bytarr(11)}

openr, lun, filename, /get_lun, /block

readu, lun, header
;stop
;print, '#1'
; Process header
; --------------

if header.site eq 'R_B' then $  ;%% Mistake in R_B CADI: 20 Hz in file header.
   header.pps = 40B
if header.site eq 'R_L' then $  ;%% Mistake in R_L CADI: 20 Hz in file header.
   header.pps = 40B
if header.site eq 'FOR' then $  ;%% Mistake in F_O CADI: 0 Hz in file header.
   header.pps = 20B
if header.site eq 'EUR' then $  ;%% Mistake in EUR CADI: 1 Hz in file header.
   header.pps = 40B
if header.site eq 'DEL'then  $ ;%% Mistake in Delaware: 1 Hz in header
   header.pps = 40B
if header.site eq 'RAN' then $  ; Mistake in some RAN CADI: 1 Hz in file header.
   header.pps = 40B


if header.noofreceivers ne 0 then nantennas = header.noofreceivers

nheights = long(header.maxheight/round(dheight)+1)

; Decode ASCII_DATETIME for conversion to CANOPUS time

month='' & day=0 & hour=0 & min=0 & sec=0 & year=0
reads, header.ascii_datetime, month, day, hour, min, sec, year, $
   format='(x,a3,x,i2,3(x,i2),x,i4)'
time_header = cantime(year, month_number(month), day,hour,min,sec)
time_hour = 3600L * (time_header/3600L)
;stop
;print, '#2'
; Read frequencies
; ----------------

freqs = fltarr(header.nfreqs)
freq_bytes = bytarr(4)
for freqx=0,header.nfreqs-1 do begin
   readu, lun, freq_bytes
   freqs(freqx) = ieee_to_float(freq_bytes)
endfor

; Form HEADER_OUT now in case it's the only thing wanted.
; But don't set fields NTIMES and TIMES, since they are unknown before the
; whole file is read.
; Set NTIMES to zero for now. Append TIMES field later.
;print, '#3'
header_out = { $
         nantennas:     long(nantennas), $
         ndops:         long(header.ndops), $
         nheights:      long(nheights), $
         ntimes:        0L, $                   ; Unknown before whole file read
         nfreqs:        long(header.nfreqs), $
         site:          header.site, $
         time:          time_header, $
         filetype:      header.filetype, $
         minheight:     header.minheight, $
         maxheight:     header.maxheight, $
         pps:           header.pps, $
         npulses_avgd:  header.npulses_avgd, $
         base_thr100:   header.base_thr100, $
         noise_thr100:  header.noise_thr100, $
         min_dop_forsave:header.min_dop_forsave, $
         dtime:         long(header.dtime), $
         gain_control:  header.gain_control, $
         sig_process:   header.sig_process, $
         dops:          ( findgen(header.ndops) - (header.ndops/2) ) / $
                           (float(header.ndops) * $
                            header.npulses_avgd / float(header.pps)), $
         heights:       dheight*indgen(nheights), $
         freqs:         freqs}

;print header vaues
print, "site = ", header.site
print, "time = ",time_header, "  time between samples =", header.dtime
print, "file type = ", header.filetype,"  signal process = ", string(header.sig_process)
print, format = '("number antennas = ",I3, "  gain control = ", I4, "   pulses per second = ",I5, "  number pulses averaged = ",I5)',$
    nantennas, header.gain_control, header.pps,header.npulses_avgd
print, format = '("number of heights = ",I4,"  minheight = ", I5, "  maxheight = ",I5)',nheights, header.minheight, header.maxheight
print, format = '("number of samples for Doppler = ", I4,"  min no Dopplers for save = ", I3)',header.ndops, header.min_dop_forsave
print, format = '("basethresholdx100 = ",I5, "  noise thresholdx100 = ", I5)', header.base_thr100, header.noise_thr100
print, format = '("number of frequencies = ", I4)',header.nfreqs
if header.nfreqs gt 5 then print, format = '("first few frequencies = ",6E13.3)',freqs(0),freqs(1),freqs(2),freqs(3),freqs(4),freqs(5)$
 else for i = 0,header.nfreqs-1 do print, format='("freq #",I2,"  = ",E13.3)',i,freqs(i)
;stop

if keyword_set(onlyheader) then begin
   close, lun
   free_lun, lun
   return, header_out
   endif
;print, 'header_out =',header_out
;stop
; Create variables to read and store data from file
; -------------------------------------------------

; Create DOPBINS, a vector of structures.
; If ONLYX, each element has only a field X.
; Otherwise, each element has fields X and IQ.

; X structure holds indices to time, freq, height, doppler
;dopbin = {cadi_dopbrf_stu, x: {cadi_dop_x_stu, t:0, f:0B, h:0, d:0B} }
;stop
dopbin = {cadi_dopbrf_stu, x: {cadi_dop_x_stu, t:0, f:0, h:0, d:0B} }	;change freq to integer
;print, 'here'
if not keyword_set(onlyx) then $
   dopbin = $
      create_struct(name='cadi_dop_stu', dopbin, 'IQ', bytarr(2,nantennas) )
dopbins = replicate(dopbin, max_ndopbins)

if header.filetype eq 'I' then $
   max_nfrebins = header.nfreqs $       ; One time in file; probably ionogram
else $
   ; Many times in file
   max_nfrebins = min([ max_ntimes * header.nfreqs, max_ndopbins ])

frebins = replicate({cadi_frebin_stu, $
   ;x:  {cadi_fre_x_stu, t:0B, f:0B}, $
   x:  {cadi_fre_x_stu, t:0B, f:0}, $	;change frequency f to integer
   gain_flag:     0B, $
   noise_flag:    0B, $
   noise_power10: 0}, $
      max_nfrebins)

times = lonarr(max_ntimes)
;print, '#4'
flag = 0B
hflag = 0
time_min = 0B
time_sec = 0B
noise_flag = 0B
noise_power10 = 0
ndops_oneh = 0B
dop_flag = 0B
iq_bytes = bytarr(2,nantennas)

;print, 'no of antennas =',nantennas
; Initialize indices
;stop
timex = -1L
freqx = header.nfreqs-1
dopbinx = -1L
frebinx = -1L

if keyword_set(inform) then print, 'Processing time ...'

; Read body of file
; -----------------

; A new record begins. Read time.
;print, '#5'
next_time:      ; This is NEXT_RECORD in data collection program
   if timex ge max_ntimes-1 then begin
      message, /inform, 'Too many times (records) in file'
      goto, close_and_quit
      endif
   ;stop
   readu, lun, time_min
   if time_min eq 'FF'XB then goto, temp        ;%% Extra FF at end of file
;print,'time = ',time_min
   readu, lun, time_sec, flag   ; FLAG is Gainflag for first freq

   timex = timex + 1
   times(timex) = time_hour + 60L*time_min + time_sec
   if keyword_set(inform) then print, timex, format='($,i5)'

   for freqx=0,header.nfreqs-1 do begin
;print, 'freqx = ',freqx
      ; Read data for this frequency.
      ; Gainflag was already read as FLAG in the most recent height loop, or
      ; when the time was read at the start of NEXT_TIME.

      readu, lun, noise_flag, noise_power10     ; 3 bytes
      frebinx = frebinx + 1
      frebins(frebinx).x = {cadi_fre_x_stu, timex, freqx}
      ;stop
      frebins(frebinx).gain_flag = flag
      frebins(frebinx).noise_flag = noise_flag
      frebins(frebinx).noise_power10 = noise_power10
      htlimitmin = ht_limitmin/3
      htlimitmax = ht_limitmax/3
      ;print,htlimit
 ;stop
      next_height:
         ; FLAG read here could have any of three meanings:
         ;    - height index of next height (FLAG lt E0)
         ;    - gainflag of next freq       (FLAG between E0 and EF)
         ;    - end of record (time) flag   (FLAG eq FF)
         readu, lun, flag
         ;print
         ;print,'flag =', flag
         ;if time_min eq 48 and flag gt 224 then print, 'bad flag = ', flag
         ;if time_min eq 48 and freqx eq 170 then print, 'flag values for freq  = 170,',flag
         if flag ge 'E0'XB then goto, freq_done
;print,'#6a'
         ; Start a new height

         readu, lun, ndops_oneh ; TNUM in data collection program
         ;if (time_min eq 48)and(freqx gt 150) then print, 'frequency = ',freqx,' height = ',flag,' number dopplers = ', ndops_oneh

         ; if ndops_oneh > 128 this is a flag for extended height so need to do height adjustment
         ;print, ndops_oneh
         hflag = fix(flag)
  ;print, 'height = ',flag
         if ndops_oneh ge 128 then begin
         ndops_oneh = ndops_oneh - 128B
         hflag = hflag + 200
         endif
  ;print,'ndops oneh =',ndops_oneh
;print, '#6b'
         if keyword_set(tfh) then begin
;print, '#6b1'
     ;stop
            ; Read past spectra for this height. FLAG is height index.
            if ((hflag lt htlimitmax) AND (hflag gt htlimitmin)) then begin
            ;if hflag gt 66 then begin ;way to restrict heights to > 200 km
             dopbinx = dopbinx + 1
             if dopbinx ge max_ndopbins then begin
               message, /inform, 'Too many doppler bins in file'
               goto, close_and_quit
             endif
             dopbins(dopbinx).x = {cadi_dop_x_stu, timex, freqx, hflag, 0}
            endif ;hflag within htlimit
            ;print,ndops_oneh
            ;print,'6b2a',ndops_oneh
            for dopx=0,ndops_oneh-1 do begin
               readu, lun, dop_flag, iq_bytes
              ;if time_min eq 48 and freqx eq 170 and flag eq 82 then print,'dopx =',dopx, ' dop flag =', dop_flag, ' iq bytes = ', iq_bytes
            endfor
            ;print,'#6b2b'
           ; endif ;hflag gt 66
            endif $ ;key_word set tfh
         else $

            ; Read spectra for this height. FLAG is height index.
; if time_min eq 48 and freqx eq 170 and flag eq 82 then begin
;       ;print,'dopx =',dopx, ' dop flag =', dop_flag, ' iq bytes = ', iq_bytes
;    for i = 0,40 do begin
;       readu, lun, flag
;       print, i, flag
;    endfor
;       ;print, 'flg = ',flag
;  endif

            for dopx=0,ndops_oneh-1 do begin
               readu, lun, dop_flag, iq_bytes

            if ((hflag lt htlimitmax) AND (hflag gt htlimitmin)) then begin
            ;if hflag gt 66 then begin ;way to restrict heights to > 200 km
            ;print, hflag
                 dopbinx = dopbinx + 1
               if dopbinx ge max_ndopbins then begin
                  message, /inform, 'Too many doppler bins in file'
                  goto, close_and_quit
                  endif
               dopbins(dopbinx).x = {cadi_dop_x_stu, timex, freqx, hflag, dop_flag}
               if not keyword_set(onlyx) then $
                  dopbins(dopbinx).iq = iq_bytes
             ; endif ;hflag gt 66
              endif ; hflag within htlimit
            endfor ; dopx

         goto, next_height
   freq_done:
;stop
      endfor ; freqx
;print, '#7'
; stop
   if flag ne 'FF'XB then begin
      message, /inform, 'FF byte not found after last frequency of record'
      goto, close_and_quit   ;sometimes get error here, continue anyways?
      endif

;         readu, lun, flag
;         print,'1 flag =', flag
;readu, lun, flag
;         print,'2 flag =', flag
;readu, lun, flag
;         print,'3 flag =', flag
;readu, lun, flag
;         print,'4 flag =', flag
;readu, lun, flag
;         print,'5 flag =', flag
;readu, lun, flag
;         print,'6 flag =', flag
;stop


   if not eof(lun) then goto, next_time         ;%% Double FF at EOF sometimes?

temp:   ;%% Needed to process extra FF at end of some files
if keyword_set(inform) then print, ' '

close, lun
free_lun, lun

; File has been read
; ------------------
 ;stop
ntimes = timex + 1
ndopbins = dopbinx + 1
nfrebins = frebinx + 1

if ntimes gt 0 then $
   times = times(0:ntimes-1) $
else $
   times = -1L

if ndopbins gt 0 then $
   dopbins = dopbins(0:ndopbins-1) $            ; Trim unused elements
else $
   dopbins = -1L

if nfrebins gt 0 then $
   frebins = frebins(0:nfrebins-1) $
else $
   frebins = -1L

; Decode Doppler bin flags
; ------------------------

if (ndopbins gt 0) and (not keyword_set(tfh)) then begin

   ; Very occasionally got a bad DOP_FLAG in FHT files
   if howmany((dopbins.x.d lt 0) or $
              (dopbins.x.d ge header.ndops)) gt 0 then begin
      message, /inform, 'Bad DOP_FLAG'
      goto, quit
      endif
;stop
   half_ndops = byte( long(header.ndops) / 2 )
   w_lo = where(dopbins.x.d lt half_ndops, lo_count)
   w_hi = where(dopbins.x.d ge half_ndops, hi_count)
   if lo_count gt 0 then dopbins(w_lo).x.d = dopbins(w_lo).x.d + half_ndops
   if hi_count gt 0 then dopbins(w_hi).x.d = dopbins(w_hi).x.d - half_ndops
   endif

; Complete the output variables and return
; ----------------------------------------

;%% header_out = create_struct(header_out, 'TIMES', times)      ; Append TIMES
;%% header_out.ntimes = long(ntimes)

form_result:
;print, 'stu'
header_exists = (datatype(header_out) eq 'STU')

;%% if header_exists then $
;%%    header_out = create_struct(temporary(header_out), 'DIR', '', 'FILE_EXT', '')

; Previously I had appended TIMES before FORM_RESULT:, and DIR and FILE_EXT
; after. But if TIMES had one element, the second CREATE_STRUCT would convert
; it to a scalar!
;stop
if header_exists then begin
   header_out = create_struct(temporary(header_out), $
      'TIMES', times,   'DIR', '',   'FILE_EXT', '')
   header_out.ntimes = long(ntimes)
   endif

if datatype(frebins) eq 'STU' then $
   nfrebins = n_elements(frebins) $
else $
   nfrebins = 0L

if datatype(dopbins) eq 'STU' then $
   ndata = n_elements(dopbins) $
else $
   ndata = 0L
;print, 'ndata = ', ndata
if keyword_set(onlyx) then $
   data_type = 'X' $
else $
   data_type = 'IQ'
;stop
return, create_struct( $
   'HEADER_EXISTS', header_exists, $
   'NFREBINS', nfrebins, $
   'NDATA', ndata, $
   'DATATYPE', data_type, $
   'HEADER', header_out, $
   'FREBINS', frebins, $
   'DATA', dopbins)

; Error handling
; --------------

handle_ioerror:
   message, /inform, 'I/O error reading file ' + filename
close_and_quit:
   close, lun
   free_lun, lun
quit:
   error = !true
   dopbins = -1L
   header_out = -1L
   frebins = -1L
   goto, form_result

end



;+
; NAME:
;      CADI_READ_DIR
; PURPOSE:
;       Read CADI data into a structured variable.
; CALLING SEQUENCE:
;       Result = CADI_READ_DIR( DIR=DIR, EXT=EXT, TIMERANGE=TIMERANGE )
;
;       VAX/VMS example:
;          To read MD1 data from 7 to 10 UT on 10-Nov-1994, for the first
;          sounding frequency only:
;
;          CADI = CADI_READ_DIR( DIR='USER$DISK:[GRANT.CADIDATA]', EXT='MD1', $
;             TIMERANGE=CANTIME(1994,11,10,7)+[0,3*3600L], FR=[0,0], /INF )
;
;       WINDOWS example:
;          To read MD1 data from 7 to 10 UT on 10-Nov-1994, for all sounding
;          frequencies but discarding all Doppler and IQ information:
;
;          CADI = CADI_READ_DIR( DIR='C:/GRANT/CADIDATA/', EXT='MD1', $
;             TIMERANGE=CANTIME(1994,11,10,7)+[0,3*3600L], /ONLYX, /TFH, /INF )
;
; KEYWORD PARAMETERS:
;    Mandatory:
;       DIR: Scalar string with name of directory holding data files, ready to
;          prepend to filename. Mandatory.
;       FILE_EXT: Extension of data files, without preceding period.
;          May be MDn, FHT or FFI (or anything else if SAVE is set). Mandatory.
;       TIMERANGE = Two-element vector of CANOPUS times, giving start and end
;          times of interval for which data is to be returned. Mandatory unless
;          ONLYHEADER is set.
;    Optional:
;       SAVE: If present and nonzero, then the files are assumed to be IDL
;          SAVE files which need only be restored. Otherwise the files are
;          assumed to be raw CADI DATA files and must be of type MDn, FHT or
;          FFI.
;       ONLYX: If present and nonzero, only the X field of CADI.DATA is
;          returned, not the IQ field. If this option is set then file reading
;          is much faster and memory use is much less, and so this option is
;          recommended if the IQ data is not required, e.g. for h't plots.
;          ONLYX set is the default if TFH is set.
;       TFH: If TFH is present and
;          nonzero, only the T, F and H fields of CADI.DATA.X are returned,
;          and not the D field, and duplicate T-F-H combinations are
;          eliminated. In other words, all Doppler and IQ information in the
;          files is ignored. This suits the usual h't plot. Use of /TFH results
;          in an even smaller storage requirement than when /ONLYX is used
;          alone. If TFH is set then ONLYX defaults to non-zero. An error
;          results if TFH is set and a zero value is provided for ONLYX.
;       ONLYHEADER: If present and non-zero then only the header, formed from
;          information in the first file, is returned. It has the structure of
;          CADI.HEADER.
;       TR, FR, HR, DR: Each of these is a two-element range of indices into
;          the TIMES, FREQS, HEIGHTS, DOPS fields respectively of CADI.HEADER.
;          They may be used to select a subset of the data. They will be passed
;          to CADI_SUB by the _EXTRA keyword mechanism.
;       ERROR: Scalar, returned as nonzero if an error occurs.
;       INFORM: If present and nonzero, progress messages will be printed.
;          INFORM = 1 (i.e. /INFORM): Prints one line for each file read.
;          INFORM = 2: Prints one line for each file, plus each timex within
;             each file.
; OUTPUTS:
;       Result:
;       - If ONLYHEADER is not set:
;          A variable of CADI structure. The label 'CADI structure' is
;          used loosely: the structures are anonymous and do not always have
;          the same fields. In particular CADI.DATA always has the subfield X,
;          but may or may not have another subfield, e.g. IQ, XAP, or KVEC.
;       - If ONLYHEADER is set:
;          A variable of CADI.HEADER structure.
; RESTRICTIONS:
;       Only reads MDn, FHT and FFI files.
;
;       %% Doesn't yet handle info on frequency bins (noise levels, etc.).
;
;       %% Won't select times within a file. Presently keeps all data in
;       %% each file contributing to the time range.
; MODIFICATION HISTORY:
;       Ian Grant, UWO, 4 Oct 1994: Written.
;       GRANT, 30 Nov 1994: Added option of INFORM = 1 or 2.
;       GRANT, 11 Jan 1995: Made /ONLYX the default when /TFH set.
;       GRANT, 3 Feb 1995: Removed special handling of files with read errors,
;          since lower-level routines now return null data structures in such
;          cases.
;          Added keyword ONLYHEADER.
;       MACDOUGALL, 3Apr1996: added code to check for empty files at beginning
;          of filelist
;		JWM, 1997 Sept25: height set using keyword htlimit (can use extended height)
;       JWM, 2000 Jan30: mods of time determination to enable use for year 2000+
;		JWM, 2004 Jan 12: changed height limit to htlimitmin and htlimitmax
;-

function cadi_read_dir, dir=dir, timerange=timerange, ext=file_ext, $
   save=save, onlyx=onlyx, tfh=tfh, _extra=extra, onlyheader=onlyheader, $
   error=error_out, inform=inform , htlimitmin=htlimitmin, htlimitmax=htlimitmax ;%% , hour_list=hour_list
;stop
;MESSAGE,'Got this far',/INFORMATION
;%% Need to concatenate frebins F as well as D.

;%% Need to select times after each file read. Presently keeps all data in
;%% each file contributing to the time range.

on_error, 0
error_out = !false

if not keyword_set(htlimitmax) then htlimitmax = 510
if not keyword_set(htlimitmin) then htlimitmin = 0
if n_elements(dir) eq 0 then message, /inform, 'Must supply keyword DIR'
if n_elements(file_ext) eq 0 then message, /inform, 'Must supply keyword EXT'
;stop
hour_inform = 0
if keyword_set(inform) then $
   if inform ge 2 then $
      hour_inform = 1

; If TFH is set, then ensure ONLYX is set also

if keyword_set(tfh) then $
   if n_elements(onlyx) eq 0 then $
       onlyx = !true $
   else $
      if not keyword_set(onlyx) then begin
         message, /inform, $
            'If TFH is set then ONLYX must either be set or absent'
         goto, quit
         endif

;%% if keyword_set(tfh) then $
;%%    if not keyword_set(onlyx) then begin
;%%       message, /inform, 'If TFH is set then ONLYX must also be set'
;%%       goto, quit
;%%       endif

; Select files from directory which have the right extension and contribute to
; the time range.
;stop
;print, 'dir = ',dir
;print, 'fileextension=',file_ext
files=findfile(dir+'*.'+file_ext, count=nfiles)
s = sort(files)
;print, s
files = files(s)
;print, sort(files)
;stop
;print, 'nfiles =',nfiles
nofiles:
if nfiles eq 0 then begin
   message, /inform, 'No files '+dir+'*.'+file_ext
   goto, return_null
   endif

done = 0
while not done do begin
 openr,1, files(0)
 res = fstat(1)
 close,1
 if res.size eq 0 then begin
  nfiles = nfiles -1
  if nfiles eq 0 then goto, nofiles
  files = files(1:*)
 endif else done = 1
endwhile

; If only the header is required, read it from the first file and return it.

if keyword_set(onlyheader) then begin
   if in(strupcase(file_ext), ['FHT', 'FFI']) then $
      h = cadi_read_ft(files(0), /onlyheader, error=error) $
   else $
      h = cadi_read_md(files(0), /onlyheader, error=error)
   if error then begin
      message, /inform, $
         'Error reading file ' + files(0) + ' for header information'
      goto, quit
      endif
   return, h
   endif

; Get file times from the filenames
;stop
if n_elements(timerange) eq 0 then $
   message, /inform, 'Must supply keyword TIMERANGE'
;stop
;os_chars, enddir_char=enddir_char
enddir_char='\'
filetime_strs = strmid(files, strposlast(files(0),enddir_char)+1, 8) ; ymddhhmm
filetimes = lonarr(nfiles)
;stop
year=0 & month_char='' & day=0 & hour=0 & minute=0
for filex=0,nfiles-1 do begin
;print,'file info =',filetime_strs(filex)
   reads, filetime_strs(filex), $
      year, month_char, day, hour, minute, format='(i1,a1,3i2)'
      ;for times in years 2000+ need to adjust the year by adding appropriate increments
      if timerange(0) ge 441763200L then year = year + 10 ; years >=2000
      if timerange(0) ge 757382400L then year = year + 10 ; years >=2010
      if timerange(0) ge 1072915200L then year = year + 10 ; years >=2020
      if timerange(0) ge 1388534400L then year = year + 10 ; years >=2030
      if timerange(0) ge 1704067200L then year = year + 10 ; years >=2040
      if timerange(0) ge 2019686400L then year = year + 10 ; years >=2050
      ;stop
month_char = strupcase(month_char)
   filetimes(filex) = cantime(1990+year, (byte(month_char)-byte('A')+1)(0), $
      day, hour, minute)
   ;stop
   endfor
;stop
; -1 second at end of timerange to avoid reading a file for one time point,
; since TIMERANGE will usually extend between two exact hours.
filetime_range = ((timerange+[0,-1]) / 60L) * 60L
w = within(filetimes, filetime_range, nfiles)
if nfiles eq 0 then goto, return_null
files = files(w)
filetimes = filetimes(w)
;stop
;print, "here"
; We have the list of relevant files. Read them.
; Either RESTORE save files, read FHT/FFI files, or read MD files.

d=0 & h=0 & f=0 & error=0 & filename=0  ; Create variables so RESTORE works
for filex=0,nfiles-1 do begin
   if keyword_set(inform) then print, 'Processing file ' + files(filex) + ' ...'
;  print, 'Processing file ' + files(filex) + ' ...'
  if keyword_set(save) then $
      restore, files(filex) $   ; Yields, D, H, F, ERROR, FILENAME
   else begin
      if in(strupcase(file_ext), ['FHT', 'FFI']) then $
         d = cadi_read_ft(files(filex), error=error, $
            ;%% inform=keyword_set(inform), $
            inform=hour_inform, $
            onlyx=keyword_set(onlyx), tfh=keyword_set(tfh)) $
      else $
         d = cadi_read_md(files(filex), error=error, $
            ;%% inform=keyword_set(inform), $
            inform=hour_inform, $
            onlyx=keyword_set(onlyx), tfh=keyword_set(tfh), $
            htlimitmin = htlimitmin, htlimitmax = htlimitmax)
     ; stop
      endelse ; save
   ;%% If CADI_READ_MD/FT returns with an error, just rely on messages from that
   ;%% routine and continue. Eventually should record and return a list of
   ;%% which files had read errors.
   ;%% Removed 950203: if error then goto, skip_file
;stop
   if keyword_set(extra) then $
      d = cadi_sub_struct(d, _extra=extra)
   if filex eq 0 then $
      d_out = d $
   else $
      d_out = cadi_concat(d_out, d)
;stop
   ;%% Removed 950203: skip_file:
   ;%% if error then error_out = !true ;%% See above re CADI_READ_MD errors

   endfor ; filex

if d_out.header_exists then begin
   d_out.header.dir = dir
   d_out.header.file_ext = strupcase(file_ext)
   endif
;stop
return, d_out

; Error handling
; --------------

return_null:

   if keyword_set(onlyx) then $
      data_type = 'X' $
   else $
      data_type = 'IQ'

   return, create_struct( $
      'HEADER_EXISTS', !false, $
      'NFREBINS', 0L, $
      'NDATA', 0L, $
      'DATATYPE', data_type, $
      'HEADER', -1L, $
      'FREBINS', -1L, $
      'DATA', -1L)

quit:
   error_out = !true
   return, -1L

end
