;+
; NAME:
;       CADI_HVSUMM
; ORIGINS:
;      Adapted from CADI_HTSUMM by Ian Grant
; PURPOSE:
;       Generate summary plot of one day's h't and velocity from CADI data files.
;       Designed for MD and FHT data formats.
; CALLING SEQUENCE:
;       CADI_HVSUMM, DIR=DIR, DATESTR=DATESTR, EXT=EXT
; INPUTS:
;       DIR = Scalar string containing directory in which data is located.
;          Syntax ready to prepend onto filename e.g. 'D:\CADIDATA\'.
;          Default is ''.
;       DATESTR = string holding date of data as 6 ASCII digits YYMMDD.
;          Default is to take date from header of first .ext file in directory.
;       EXT = scalar string containing extension of data files, without period.
;          Default is 'MD1'.
; KEYWORD PARAMETERS:
;       INFORM: If present and nonzero, progress messages are printed.
;       SUBPLOT_NHOURS = Integer number of hours in each panel. Default 6.
;       NSUBPLOTS = Integer number of panels. Default 24/SUBPLOT_NHOURS.
;       START_HOUR = Integer hour at start of first panel. Default 0.
;       FREQX = scalar integer index into list of sounding frequencies in file
;          header. Selects frequency to be plotted. Default is 0.
;       ERROR = integer scalar. Returned as true if an error occurred, else
;          false.
; RESTRICTIONS:
;       %% Plots all files in directory, even if outside range of timeaxis.
;       %% Single hour is too high to fit on 5:4 plot. Use PORTRAIT in that
;          case?
; SIDE EFFECTS:
;       Generates a plot file.
; MODIFICATION HISTORY:
;		23 Sept 1997 by JWM added maxheight in call to hvsumm
;  		28 Jan 2000 by JWM, modified for Y2K
;-

pro cadi_hvsumm, dir=dir, datestr=datestr_in, ext=file_ext, $
   inform=inform, error=error, $
   subplot_nhours=subplot_nhours, nsubplots=nsubplots, start_hour=start_hour, $
   freqh=freqh, freqv=freqv, verscale = verscale, horscale = horscale, panelselect = panelselect

;on_error, 2

psav = !p
xsav = !x
ysav = !y

; Set up plot now to be ready for any 'Aborted' message.

erase


;os_chars, wild_1char=wild_1char, enddir_char=enddir_char

if not keyword_set(dir) then dir = ''
if not keyword_set(file_ext) then file_ext = 'MD1'
if not keyword_set(freqh) then freqh = 0
if not keyword_set(freqv) then freqv = 0

;%% ; Find the first file with the right extension in the directory,
;; read its header, and extract some information from the header.
;
;files=findfile(dir+'*.'+file_ext, count=nfiles)
;if nfiles eq 0 then begin
;   error_message = 'No files '+dir+'*.'+file_ext
;   goto, error_return
;   endif
;;%% h = cadi_read_md(files(0), /onlyheader, error=error)
;if in(strupcase(file_ext), ['FHT', 'FFI']) then $
;   h = cadi_read_ft(files(0), /onlyheader, error=error) $
;else $
;   h = cadi_read_md(files(0), /onlyheader, error=error)
;if error then begin
;   error_message = 'Error reading first file for header information'
;   goto, error_return
;   end

; Get header information (CADI_READ_DIR get it from first file)

h = cadi_read_dir(dir=dir, ext=file_ext, /onlyheader, error=error)
if error then begin
   error_message = 'Error reading first file for header information'
   goto, error_return
   endif
;print, 'h =',h
max_height = h.maxheight
;stop
dh = median(difference( h.heights ))         ; Typical height step; pixel height

;; Determine default pixel width DT.

;;%% dt = h.dtime
if strupcase(file_ext) eq 'FHT' then $
   dt = h.nfreqs * h.dtime $
else $
   dt = h.dtime

; Input date from keyword or from first file

if not keyword_set(datestr_in) then begin
  candate = canmidnight(h.time)    ; Date from time in first file
endif else begin
   datestr = datestr_in ; Unnecessary?
   reads,datestr,year,month,day,format='(3i2)'   ; Date from keyword
   ;print, 'year = ',year
   if year lt 54 then year = year + 100
   candate=cantime(1900+year,month,day,0,0,0)
   endelse

;if not keyword_set(subplot_nhours) then subplot_nhours = 24L
if not keyword_set(subplot_nhours) then subplot_nhours = 12L
if not keyword_set(nsubplots) then nsubplots = 24/subplot_nhours
if not keyword_set(start_hour) then start_hour = 0L
civil_time, year, month, day, daynum, hour, minute, second, candate ; For DAYNUM


;rx = 0L         ; Index of receiver to be plotted
!p.noerase = 1

; Set POS_REGIONS(1) and POS_IN_REGIONS(1) so as to position bottoms of
; subplots to give one hour a square aspect ratio.

pos_regions = [0.1,-1,0.95,0.93]
pos_in_region = [0.0, -1, 1.0, 1.0]
subplot_ynorm = $
   ((pos_regions(2) - pos_regions(0)) / subplot_nhours * aspect(/device))
subpltgap_ynorm = 0.05   ; Space for hour labels of subplots, norm y distance
pos_regions(1) = pos_regions(3) - ((subplot_ynorm + subpltgap_ynorm) *nsubplots)
pos_in_region(1) = subpltgap_ynorm / (subplot_ynorm + subpltgap_ynorm)
if pos_regions(1) lt 0.02 then $
   message, /inform, 'Too many subplots (' + strim(nsubplots) + $
      ') for this length of subplot (' + strim(subplot_nhours) + ' hours)'

; Start plot

;%% repeat begin ; until hardcopy()
;if !d.name eq 'PS' then begin
;   device, bits_per_pixel=1
;   back_color = 255b    ; White
;   fore_color = 0b      ; Black
; endif else if !d.name eq 'PCL' then begin
;      back_color = 255b ; White
;      fore_color = 0b   ; Black
; endif else begin
;   back_color = 0b      ; White
;   fore_color = 255b    ; Black
;   endelse
;;%% erase
;;%% time
;
; Set up subplot regions

;region_multi, /init, [1,nsubplots], pos=pos_regions ;, $
;   title='CADI ' + h.site + " h't    " + $
;   strcantime(/date,candate) + '    Day ' + string(daynum, format='(i3)') + $
;   '    ' + string(h.freqs(freqx)/1e6, format='(f6.3)') + ' MHz'

; Loop through subplots

;!p.multi = [0,nsubplots]

for subplotx=0L,nsubplots-1 do begin
;   ; Hours on axis, including ends
;   subplot_hours = $
;      start_hour + subplot_nhours * subplotx + lindgen(subplot_nhours+1)
;   ; Axis range (seconds)
;   plot_timerange = ends(subplot_hours)*3600L
;
;   ; Define axes of subplot without drawing
;
;   region_multi, subplotx
;   region_position, pos_in_region
;   plot, /nodata, [0], [0], $
;      xrange=plot_timerange, xstyle=5, $
;      yrange=[0,max_height], ystyle=5
;
;   ; Read and plot data for this subplot
;
;   cadi = cadi_read_dir(dir=dir, timerange=candate+plot_timerange, $
;      ext=file_ext, /tfh, fr=[freqx,freqx], error=read_error, $
;      inform=keyword_set(inform))
;   if cadi.ndata gt 0 then begin
;
;      ; Find smallest rectangle containing all set pixels
;
;      min_xt = min(cadi.data.x.t, max=max_xt)
;      min_xh = min(cadi.data.x.h, max=max_xh)
;      tdim = max_xt - min_xt + 1
;      hdim = max_xh - min_xh + 1
;
;      ; Create an image of this smallest rectangle and plot it
;
;      z = 0             ; Deallocate memory
;      z = make_array( tdim, hdim, /byte)
;      if (size(z))(0) ne 2 then $
;         ; Ensure Z is 2D even if one or both dimensions are 1
;         z = reform( temporary(z), tdim, hdim )
;      z( cadi.data.x.t-min_xt, cadi.data.x.h-min_xh ) = 1B
;      cadi_tvft, z, $
;         cadi.header.times(min_xt:max_xt) - candate, $
;         cadi.header.heights(min_xh:max_xh), $
;         dx=dt, dy=dh, fore_color=fore_color, back_color=back_color
;      endif ; cadi.ndata gt 0
;
;   ; Plot axes for subplot
;
;   if subplotx eq nsubplots-1 then $
;      xtitle = 'Hour (UT)' $
;   else $
;      xtitle = ' '
;   plot, /nodata, [0], [0], $   ; Plot axes
;      xrange=plot_timerange, xstyle=1, $
;      xticklen=!p.ticklen / (!p.region(3)-!p.region(1)), $
;      xticks=subplot_nhours, xtickv=subplot_hours*3600L, xminor=6, $
;      xtickname=string(subplot_hours, format='(i2.2)'), xtitle=xtitle, $
;      yrange=[0,max_height], ystyle=1, ytitle='Height (km)', yminor=5, $
;      yticklen=-0.6*!p.ticklen

if subplot_nhours eq 12 then tr = candate + [subplotx*12*3600L, (subplotx+1)*12*3600L] $
 else tr = candate + [subplotx*24*3600L, (subplotx+1)*24*3600L]
;print,'tr =',tr
; Get header information (CADI_READ_DIR get it from first file)
;stop
h = cadi_read_dir(dir=dir, ext=file_ext, timerange = tr, /tfh)
if not h.header_exists then goto, finishplot
v = cadi_read_dir(dir=dir, ext=file_ext, timerange = tr, fr = [freqv,freqv])
;print, 'v = ',v
vel = cadi_iq2vel(v)
vel_edit = cadi_vel_edit(vel, nsources = 1, azi = 10000, hor = 1000, ver = 1000, /drop)
;vel_edit = cadi_vel_edit(vel, nsources = 1, azi = 360, hor = 1000, ver = 1000, /drop)
if error then begin
   error_message = 'Error reading first file for header information'
   goto, error_return
   endif
;stop
if not keyword_set(verscale) then verscale = 500
if not keyword_set(horscale) then horscale = 2000

if ((file_ext eq 'md4')or (file_ext eq 'md3')) then maxht = 1020 else maxht = 510
;maxht = 510 ; suppress the upper height ranges: need to also modify CADI_HTV if want to
			; plot the full 1020 km range
;stop
if panelselect eq 'XY' then $
   cadi_htv, h, vel=vel_edit, timerange=tr, panel='fXYVN',fr=freqh, max_height = maxht, $
   /inf, ver = verscale,$
   hor = horscale $
else cadi_htv, h, vel=vel_edit, timerange=tr, panel='fAHVNahv',fr=freqh,$
      max_height = maxht, /inf, $
      ver = verscale, hor = horscale

finishplot:

   endfor ; subplotx
time
;region_multi, /reset
;%% endrep until hardcopy()

!p.noerase = 0
!p = psav
!x = xsav
!y = ysav

error = !false
return

error_return:
error = !true
if not keyword_set(error_message) then error_message = ''
close, /all
;region_multi, /clear
message, /inform, 'Aborted. '+error_message
xyouts, 0.3, 0.04, /norm, "Aborted h't summary plot for " + $
   datestr_in + ', ' + dir + '*.' + file_ext + $
   '!c' + error_message
return

end
