;+
; NAME:
;       CADI_HTFSUMM
; PURPOSE:
;       Generate summary plot of ionograms as h't with freq -> colour
;           from CADI data files.
;       Designed for MD and FFI data formats.
; CALLING SEQUENCE:
;       CADI_HTFSUMM, DIR=DIR, TIMERANGE=TIMERANGE, EXT=EXT
; INPUTS:
;       DIR = Scalar string containing directory in which data is located.
;          Syntax ready to prepend onto filename e.g. 'D:\CADIDATA\'.
;          Default is ''.
;       TIMERANGE = canopus timerange
;       EXT = scalar string containing extension of data files, without period.
;          Default is 'MD0'.
; KEYWORD PARAMETERS:
;       INFORM: If present and nonzero, progress messages are printed.
;       ERROR = integer scalar. Returned as true if an error occurred, else
;          false.
; SIDE EFFECTS:
;       Generates a plot file.
; MODIFICATION HISTORY:
;       John MacDougall, UWO, 30 Nov 1995.
;	JWM 2001 June 21 to look at multi ionogram data added keyword contour
;		and to manipulate the data added keywords: heightrange (2 element heightrange)
;		freqrange (2 element frequency range), mps (set to generate multi ps output)
;	jwm 2004 april 1: mods for widgets, added colour keyword
;	jwm 2004april 4: added tfplot keyword to do time-frequency plots
;-

pro cadi_htfsumm, dir=dir, timerange=timerange, ext=file_ext, $
   inform=inform, error=error, contour = contour, heightrange = heightrange, $
   freqrange = freqrange, mps = mps, colour = colour, tfplot = tfplot, saveFT = saveFT,$
   scale = scale

on_error, 0 ;2

;multiday = 1
;if keyword_set(multiday) then begin
;print, 'starting multiday'
;  !p.multi = [0,1,5]
;  daynumber = 0
;endif

psav = !p
xsav = !x
ysav = !y
pticklen = !p.ticklen

; Set up plot now to be ready for any 'Aborted' message.

;erase
;time
plot_time = 0.0
dt = 0.0

if not keyword_set(file_ext) then file_ext = 'MD0'
if not keyword_set(dir) then dir = ''

; Get header information (CADI_READ_DIR get it from first file)
;anotherday:

h = cadi_read_dir(dir=dir, timerange = timerange, ext=file_ext, /onlyheader, error=error)
if error then begin
   error_message = 'Error reading first file for header information'
   goto, error_return
   endif
dt = float(h.dtime)/60
; Get DAYNUM for plot title

; Input date from first file
candate = canmidnight(h.time)    ; Date from time in first file
can_date = h.time
civil_time, year, month, day, daynum, endhour, minute, second, timerange(1)
endday = day
endmonth = month
;endhour = hour
endmin = minute
endsec = second
civil_time, year, month, day, daynum, starthour, minute, second, timerange(0)
if ((endday gt day)or(endmonth ne month)) then endhour = endhour + 24
endtimesec = endsec + endmin*60 + endhour*3600
starttimesec = second+minute*60+starthour*3600

;plot_timerange = timerange
plot_timerange = [starthour*3600L,endhour*3600L]
if plot_timerange(1) - plot_timerange(0) le 6*3600 then $
   reftime = canmidnight(plot_timerange(0)) $
else $
   reftime = canjan0(plot_timerange(0))
plot_timerange = plot_timerange - reftime

plot_timerange = [starttimesec,endtimesec]
;stop
rx = 0L         ; Index of receiver to be plotted
;!p.noerase = 1
;!p.ticklen = 0.04
;charsize = 0.8
;pos_regions = fltarr(4,2)
;pos_regions(0,0) = [0.05, 0.55, 1.0, 0.93]
;pos_regions(0,1) = pos_regions(*,0) - 0.48*[0, 1, 0, 1]
;pos_in_region = [0, 0, 1, 1]
;def_ticknames = replicate('',20)
;blank_ticknames = replicate(' ',20)

; Start plot
;if keyword_set(multiday) then goto, skippmulti
if keyword_set(colour) then !p.multi = [0,1,2] else !p.multi = 0 ;set to default 1 plot per page
;skippmulti:

back_color = 255b      ; black
!p.background = 255b
fore_color = 0b    ; white
!p.color = 0b
if !d.name eq 'PS' then begin
   device, bits_per_pixel=8
   back_color = 255b    ; White
   fore_color = 0b      ; Black
endif else if !d.name eq 'PCL' then begin
      ;device, /inches, xsize = 10, ysize = 3
      !P.multi = [0,0,2,0,0]
      print, 'pcl mode'
      back_color = 255b ; White
      fore_color = 0b   ; Black
endif else if keyword_set(colour) then begin
   back_color = 255b    ;white
   !p.background = 255b
   fore_color = 0b		;black
   !p.color = 0b
endif
erase
prev_timex = -1
prev_panelx = -1

;get frequency information
 freqarr = h.freqs
 if keyword_set(freqrange) then begin
 ;print, 'freq range =',freqrange(0),freqrange(1)
   ;find max and min frequency numbers
   select = where(freqarr le freqrange(1),maxfno)
   maxfno = maxfno - 1
   select = where(freqarr lt freqrange(0),minfno)
  ; print, 'freq range nos =',minfno,maxfno
 endif else begin	;no freqrange set
  maxfno = h.nfreq - 1
  minfno = 0
 endelse

if keyword_set(heightrange) then begin
htmin = heightrange(0)
htmax = heightrange(1)
if htmax gt 510 and file_ext ne 'md4' then htmax = 510
endif else begin
htmin = 0
htmax = 510
endelse


; Plot title above postition for top panel
if keyword_set(colour) then pos = [0.05,0.08,0.93,0.95] else pos = [0.05,0.1,0.98,0.95]
if keyword_set(tfplot) then begin
   partlabel = ' time-freq '
   yaxisrange = [floor(freqrange(0)/1.0e6),ceil(freqrange(1)/1.0e6)]
   ylabel = 'frequency, MHz'
endif else begin
   partlabel = ' ht & freq '
   yaxisrange = [htmin,htmax]
   ylabel = 'group range, km'
endelse
if not keyword_set(contour) then plot, [0,3],[0,510],/nodata, xrange = [starthour*3600,endhour*3600], xstyle = 5, $
     yrange = yaxisrange, ystyle = 1, position = pos, xticks = 6, xminor = 2,yminor = 5,$
   title='CADI ' + h.site + partlabel + strcantime(/date,can_date) + $
   '    Day ' + string(daynum, format='(i3)'),$
     xtitle = 'Hour (UT)', $
      ytitle = ylabel

;print, 'here', starthour, endhour
if (endhour eq 0) then endhour = 24
; Loop through hours
;stop
noofhours = endhour - starthour
speckles = fix(24/noofhours)
nooftimes = 3600*(endhour - starthour)/h.dtime
timearr = fltarr(nooftimes)

heightarr = fltarr(3600*(endhour - starthour)/h.dtime, h.nfreqs)
;stop
for hourx = starthour,endhour-1 do begin
   timerange_hour = candate + 3600L*hourx + [0,3599L]
   print, 'reading data for hour = ', hourx
   if keyword_set(tfplot) then valuetfh = 0 else valuetfh = 1
   cadi = cadi_read_dir(dir=dir, timerange=timerange_hour, htlimitmax = htmax, htlimitmin = htmin,$
      ext=file_ext, TFH = valuetfh, error=read_error, inf=keyword_set(inform) )
   if read_error then begin
      message, /inform, $
         'Error reading data in time range '+strcanrange(timerange_hour)
      goto, end_hourx_loop
   endif
   ;stop
   if not cadi.header_exists then $
      goto, end_hourx_loop      ; No soundings for this hour
   h = cadi.header
   d = cadi.data
   ;stop
   ;dh = median(difference( h.heights )) ; Typical height step; pixel height
   max_i = round(2*95*dt)

   for htimex=0,h.ntimes-1 do begin
      civil_time, year, month, day, daynum, hour, minute, second, h.times(htimex)
     ; if (minute gt 2) then goto, end_hourx_loop
     goto, skipnewplot
     if not keyword_set(contour) then begin
       if (hour eq (starthour + 3)) then begin
          plot, [0,3],[0,510],/nodata, xrange = [hour,hour+3], yrange = [htmin,htmax], ystyle = 1,$
            xticks = 3, xminor = 4,yminor = 5,$
              title='CADI ' + h.site + ' ht & freq ' + strcantime(/date,can_date) + $
                '    Day ' + string(daynum, format='(i3)'),$
             xtitle = 'Hour (UT)', $
               ytitle = 'group range, km', xstyle = 5 ;no x axis info here
           starthour = hour
       endif
     endif
skipnewplot:
   plot_time = float(hour)*3600L + float(minute)*60 + float(second)
   ;timearr(htimex) = plot_time





    if cadi.ndata gt 0 then begin          ; Any echoes in this hour?
        w = where(d.x.t eq htimex, wcount)  ; Any echoes for this time?
    if wcount gt 0 then begin
      d_ionogram = d(w)
;ok check to save values into the heightarr


;print, 'wcount =',wcount, ' hour =', hour,' minutes =', minute
fpt = 1.0
tpt = 1.0
cpt = 1.0
oldf = 0
maxdb = 0.0
for countw = 0, wcount-1 do begin

if keyword_set(contour) then begin

  if keyword_set(heightrange) then begin
    if (3*d_ionogram(countw).x.h ge htmin) and (3*d_ionogram(countw).x.h le htmax) then $
      hgt = 3*d_ionogram(countw).x.h else hgt = 0
  endif else hgt = 3*d_ionogram(countw).x.h
  frq = d_ionogram(countw).x.f
  ;if hgt gt 0 then stop
  if ((heightarr(htimex,frq) eq 0)or (hgt lt heightarr(htimex,frq)))and (hgt gt 0) then heightarr(htimex,frq) = hgt
endif ;end contour
                ;re = d_ionogram(countw).IQ(0,0)
                ;im = d_ionogram(countw).IQ(1,0)
                ;if (((re*re + im*im) gt 40)AND(d_ionogram(countw).x.f lt 60)) then begin


if not keyword_set(contour) then begin

 if (((d_ionogram(countw).x.f le maxfno)and (d_ionogram(countw).x.f ge minfno))and $
    ((3*d_ionogram(countw).x.h ge htmin) and (3*d_ionogram(countw).x.h le htmax))) then begin

          ; polyfill, [plot_time,plot_time+dt/60,plot_time+dt/60, plot_time,plot_time], $
          ;      [3*d_ionogram(countw).x.h,3*d_ionogram(countw).x.h,3*d_ionogram(countw).x.h+3,$
            ;        3*d_ionogram(countw).x.h+3, 3*d_ionogram(countw).x.h],$
              ;         color = fore_color
;i = d_ionogram(countw).x.f
;i = round(2*i*dt)

;goto, cse
; xdots = randomu(seed,max_i)
   ; xd = xdots(0:i)*dt/60 + plot_time
   xd = randomu(seed,speckles)*dt*60 + plot_time  ;makes array of 15 random dots for plot_time
;ydots = randomu(seed,max_i)
  ; yd = 3*ydots(0:i) + 3*d_ionogram(countw).x.h
   if keyword_set(tfplot) then begin
        ydfi = d_ionogram(countw).x.f
        if oldf ne ydfi then begin
          oldf = ydfi
          maxdb = 0.0
        endif
        ;stop
       ;yd = fltarr(n_elements(ydf))
       ;print, 'n elements = ',n_elements(yd)
       ;for i = 0,n_elements(yd)-1 do yd(i) = h.freqs(ydf(i))/1.0e6
       ydf = h.freqs(ydfi)/1.0e6
       if ydfi lt h.nfreqs -1 then deltaf = h.freqs(ydfi+1)/1.0e6 - ydf else deltaf = ydf - h.freqs(ydfi-1)/1.0e6
       yd = randomn(seed,speckles)*deltaf/2 + ydf
       ;get magnitude of echoes
       zre = float(d_ionogram(countw).IQ(0,0))
       if (zre gt 127) then zre = zre - 256
       zim = float(d_ionogram(countw).IQ(1,0))
       if (zim gt 127) then zim = zim - 256
       zdb = 10*alog10(zre^2 + zim^2)
       if zdb gt maxdb then maxdb = zdb
       pcolor = 7*maxdb
       ;print,'IQ = ', d_ionogram(countw).IQ, ' db = ', zdb
       ;stop
   endif else  yd = 3*randomu(seed,15) + 3*d_ionogram(countw).x.h
   ;nooffreq = 95
   usersym, [-1,1,1,-1,-1],[-1,-1,1,1,-1],/fill
   if keyword_set(colour) then begin
     if keyword_set(tfplot) then pcolor = 7*maxdb else pcolor = (d_ionogram(countw).x.f - minfno)* (float(255)/(maxfno - minfno))
     if (pcolor gt 255) then pcolor = 255
;stop
     if ((plot_time eq tpt)and(yd(0) eq fpt)) then begin
        if (pcolor lt cpt) then pcolor = cpt
     endif else cpt = 0
     tpt = plot_time
     fpt = yd(0)
   ;  oplot, xd,yd,psym=3,color = 255 - pcolor;, symsize = 0.5
    oplot, xd,yd,psym=8,color = 255 - pcolor, symsize = 0.5   ;try this to get more filling
   endif else oplot, xd,yd,psym=3
 endif ;end if frequency lt
endif ;if not contour
;cse:
endfor ;countw


  endif ; wcount gt 0
endif ; cadi.ndata gt 0
endfor ; timex

   end_hourx_loop:
   endfor ; hourx





goto, skipplotsimulation
noof = 0
line = ' '
openr,2, 'c:\cdata\Es_sim_summer.dat'
readf,2,noof
print, noof
time = fltarr(noof)
for i = 0, noof-1 do begin
 readf,2, line
 time(i) = float(line)    ;note this is local time
endfor
height = fltarr(noof)
for i = 0, noof-1 do begin
 readf,2,line
 height(i) = float(line)
endfor
close,2
;print, 'summer simulation read, no of values = ', n_elements(height)
height = [height(5*360:(24*360)-1),height(0:(5*360)-1)]  ;rearrange height into UT
;print, n_elements(height)
;print, n_elements(time)
oplot,time*3600L,height, linestyle = 2, thick = 3.5,color = 0
;stop

skipplotsimulation:

;if keyword_set(contour) then begin
;   if keyword_set(freqrange) then begin
;   ;find max and min frequency numbers
;   select = where(freqarr le freqrange(1),count1)
;   select = where(freqarr lt freqrange(0),count2)
;   freqarrR = extrac(freqarr, count2,count1)
;   heightarrR = extrac(heightarr,0,count2,nooftimes,count1-count2)
;   endif else begin
;   freqarrR = freqarr
;   heightarrR = heightarr
;   endelse
;   ;smoothedZ = min_curve_surf(heightarrR,timearr,freqarrR)
;   ;freqarrR = freqarrR/1.0e6
;freqarrR = freqarrR*freqarrR/80.5  ; convert frequency to electron density
;sel = where(freqarrR lt 1.0e12,uplim)
;freqarrR = extrac(freqarrR,0,uplim)
;sze = n_elements(heightarrR(*,0))
;heightarrR = extrac(heightarrR,0,0,sze,uplim)
;      ;contour,heightarrR,timearr,freqarrR, levels = [0,99,105,110,115,120,130,140,150], c_labels = [0,1,0,1,0,1,0,1,0], /fill, $
;   ;     xtitle = 'time, hours UT',ytitle = 'frequency, MHz', $
; ;title='CADI ' + h.site + ' height versus t & freq ' + strcantime(/date,can_date)



;  if keyword_set(mps) then set_plot, 'ps'
;  for rep = 0,9 do begin
;  !p.multi = [0,3,4]
;  if keyword_set(mps) then device, filename = 'd:\plotfiles(temp)\data_nmbr'+ string(rep,format='(i3)') + '.ps'
;    for t = 0,11 do begin
;  if t eq 0 then ptitle = 'CADI ' + h.site + strcantime(/date,can_date) else $
;  ;    ptitle = 't + ' + string(5*t, format='(i3)') + 'min'
; ptitle = 't + ' + string((rep*6.0 + 0.5*t), format='(f5.1)') + 'min'
;    ;plot,freqarrR, heightarrR(t*10,*), yrange = [100,150], ystyle = 1, psym = 3,$
;plot,freqarrR, heightarrR(t + 10*rep,*), yrange = [0,510], ystyle = 1, psym = 3,$
;
;     ytitle = 'height', xtitle = 'el density/m^3',title = ptitle
;  ;for u = 0,8 do oplot, freqarrR,heightarrR(t*10+u,*), psym = 3
;  endfor ;rep
;if keyword_set(mps) then device, /close
;  ;stop
;  endfor
;endif ;end mps



;put on timescale
   xtitle = 'hours, UT'
   ;nodaylabels = 1
   ;!p.ticklen = pticklen / (pos(3) - pos(1))
   ;!x.charsize = charsize*0.75   ; Compensate CHARSIZE=0.8 in TIMESCALE
   timescale, plot_timerange(0), plot_timerange(1), title=xtitle, charsize = 1, $
      top=top, nolabels=0, tic_factor=1, nodaylabels=0
   ;!p.ticklen = pticklen
 ;stop



if keyword_set(saveFT) then begin
  ;put on boxes for clicking
  usersym, [-1.0,1.0,1.0,-1.0,-1.0],[-1.0,-1.0,1.0,1.0,-1.0],/fill   ;box usersym
  ftgraphbottom = floor(freqrange(0)/1.0e6)
  ftgraphtop = ceil(freqrange(1)/1.0e6)
  oplot,[starthour + 7200],[ftgraphtop -1],psym = 8, symsize = 5, color = 200
  print, 'bottom =',ftgraphbottom, '  timerange = ',plot_timerange
  ;xyouts, 8,4, 'step:', color = 50
  stop
  wait, 2
endif




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                        use mouse for fof2                                                        ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

if keyword_set(scale) then begin
;put in the section below to use for getting fof2 for whole or part day.
;after doing tf plot you hold down mouse button and run it along the fxf2 critical.
;If file for the day already exists new values will be added

;

;put on 'end' box
;usersym, [-1.0,1.0,1.0,-1.0,-1.0],[-1.0,-1.0,1.0,1.0,-1.0],/fill
;leftaxis = timerange(0)
polyfill,[10,40,40,10,10],[60,60,90,90,60],/fill, color = 230 ,/device
xyouts, 10,75, 'done', color = 50, /device
;
;open file for data
datafilename = 'c:\cdata\' + strmid(dir,3,8) + '.tf'
print, datafilename
openw, 1, datafilename, /append

 ;next get mouse click for what to do
 repeatMouseclick:
  CURSOR, x, y, /device, /wait
;  print,'mouse ',x,y
  if (x ge 40) and (y gt 50) and (y lt 590) then begin    ;cursor in plot region, start reading data
   dno = 0								;array pointer
   dxarr = fltarr(1440)					;frequency array
   dyarr = fltarr(1440)					;height array
   oldx = 0.0
   site = cadi_site(h.site, h.time)
   deltfcrit = 0.5*site.gyrofreq
   ;print, deltfcrit
  repeat begin
   cursor, x, y, /nowait, /data	 	;read cursor position in data coordinates
   ;if finc gt 0.2 then fs = 0.2 else fs = 0.1	;uselargersteps if wide frequency range
    if x ge oldx + 60	 then begin		;if time incremented by 1 min save the data
     dxarr(dno) = x					;in arrays dxarr
     dyarr(dno) = y 					;and dyarr
     dno = dno + 1						;increment the array pointer
     oldx = x 							;copy frequency to oldx (could just get value from array)
    endif
   endrep until !mouse.button ne 1 		;mouse button released so datascan finished
   ;stop
   sel = where(dxarr gt 0,count)
   if count gt 0 then begin
    dxarr = dxarr(sel)
    dyarr = dyarr(sel)
    oplot, dxarr,dyarr,thick = 2
   ;put data in file
    for n = 0, n_elements(dxarr)-1 do printf, 1, format = '(i8,f6.2)',dxarr(n),dyarr(n)- deltfcrit
   endif
  goto, repeatMouseclick
 endif ;mouse x,y
 close, 1
endif ;keyword tfplot

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;if color then put on frequency scale
if keyword_set(colour) then begin
  ;if tfplot then put on db scale
  if keyword_set(tfplot) then begin
    yticklabels = ['0','6','12','18','24','30','36']
    plot, [0,1],[0,1],/nodata, yrange = [0,256], ystyle = 1, xrange = [0,1],$
     position = [0.96,0.18,0.99,0.85], xstyle = 4, yticks = 6, ytickname = yticklabels, title = 'dB'
    for c = 0,255 do oplot, [0,1],[c,c],color = 255 - c, thick = 3
  endif else begin			;freq colourscale
    yticklabels = strarr(6)
    for s = 0,5 do begin
      f_n = round(float(maxfno - minfno)*s/5) + minfno
      f = h.freqs(f_n)/1.0e6  ;h.freqs(fix(float(h.nfreqs)*0.1*s))/1e6
;stop
      yticklabels(s) = string(format = '(f4.1)', f)
    endfor
    plot, [0,1],[0,1],/nodata, yrange = [0,256], ystyle = 1, xrange = [0,1],$
     position = [0.96,0.18,0.99,0.85], xstyle = 4, yticks = 5, ytickname = yticklabels, title = 'freq, MHz'
    for c = 0,255 do oplot, [0,1],[c,c],color = 255 - c, thick = 3
  endelse
endif

;if keyword_set(multiday) then begin
;  daynumber = daynumber+ 1
;  if daynumber lt 5 then goto, anotherday
;endif

;set_plot, 'win'
;stop
!p.noerase = 0
!p = psav
!x = xsav
!y = ysav

error = !false
return

error_return:
error = !true
if not keyword_set(error_message) then error_message = ''
close, /all
;region_multi, /clear
message, /inform, 'Aborted. '+error_message
xyouts, 0.3, 0.04, /norm, "Aborted ionogram summary plot for " + $
   datestr_in + ', ' + dir + '*.' + file_ext + $
   '!c' + error_message
return

end
