;+
; NAME:
;	CANTIME_IMAGES
; PURPOSE:
;	Plot stacked images against CANOPUS time.
; CALLING SEQUENCE:
;	CANTIME_IMAGES, Z, TIMES, Y
; INPUTS:
;	Z = 3D array of images. Dimensions (ntimes, ysize, nimages).
;	TIMES = Vector of length ntimes of CANOPUS times.
;	Y = Vector of length ysize of Y axis values.
; KEYWORD PARAMETERS:
;	POSITION = Vector of length 4. Position coordinates of
;		rectangle enclosing all images, excluding labels, relative to
;		the plot region.
;	TITLE = String containing title for page.
;	YTITLES = Vector of length ysize of strings. Y-title for each image.
;	ZTITLES = Vector of length ysize of strings. Legend titles for images.
;	NLEVELS_ARR = Vector of length nimages. Number of levels for each image.
;		If NLEVELS_ARR is given then LEVELS_ARR must also be given.
;		If absent or zero then the user is prompted for it and the
;		entered value is passed back to the calling program.
;	LEVELS_ARR = 2D array, dimensions (>= MAX(NLEVELS_ARR), nimages).
;		LEVELS_ARR(*,0:NLEVELS_ARR(I)-1) is levels vector for image i.
;		If LEVELS_ARR is given then NLEVELS_ARR must also be given.
;		If absent or zero then the user is prompted for it and the
;		entered value is passed back to the calling program.
;
;		The number of levels (call it NLEVELS) is related to the
;		number of colours plotted (call it NCOLORS) as follows.
;		   For IMAGETYPE EQ 'c' (filled contours),
;		NCOLORS = NLEVELS + 1, with the first (last) colour used for
;		values below (above) the first (last) level, and colour i used
;		where level(i-1) < value < level(i) for i=1,...,NLEVELS-1.
;		  For IMAGETYPE EQ 'p' (pixels),
;		NCOLORS = NLEVELS - 1, with the first (last) colour used for
;		values below (above) the first (last) level, and colour i used
;		where level(i) < value < level(i+1) for i=1,...,NLEVELS-2.
;	IMAGETYPE = 'c' for filled contours, 'p' for rectangular pixels.
;	CONTOURS: If present and non-zero, filled contours will be used.
;	PIXELS:   If present and non-zero, rectangular pixels will be used.
;	Only one or none of IMAGETYPE, CONTOURS or PIXELS may be set.
;       TIMENOLABELS: If present and non-zero, the time axis is not labelled.
; SIDE EFFECTS:
;	Generates graphics.
; PROCEDURE:
;	For contours, calls CONTOUR, CONTOUR_PREFILL and POLYCONTOUR.
;	For pixels, calls TVXY.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 5 Nov 1991: Written in IDL2.
;	IFG 14 Dec 1992: Added ON_ERROR, 2.
;	IFG 27 May 1993: Added keyword ZTITLES.
;	   Fixed legend position to be just right of plot rather than near
;	   right edge of device area.
;	GRANT, 14 Sep 1993: Changed from POLYCONTOUR to CONTOUR, /FILL.
;	   Added keywords CONTOURS, PIXELS.
;	GRANT, 28 Sep 1993: Changed REGION_MULTI calls to suit new definition
;	   of POSITION argument. Default number of colors made closer to a max
;	   of 14.
;	GRANT, 21 Dec 1993: Added TIMENOLABELS keyword.
;-

pro cantime_images, z, times, y, $
   position=pos_regions, title=title, ytitles=ytitles, ztitles=ztitles, $
   nlevels_arr=nlevels_arr, levels_arr=levels_arr, $
   imagetype=imagetype_in, contours=contours, pixels=pixels, $
   timenolabels=timenolabels

on_error, 2
psav = !p
xsav = !x
ysav = !y

case howmany( [ keyword_set(imagetype_in), keyword_set(contours), $
                keyword_set(pixels) ]) of
   0: imagetype_in = reply_char('Filled [C]ontours or [P]ixels?', 'cp', 'c')
   1: if not keyword_set(imagetype_in) then begin
         if keyword_set(contours) then $
            imagetype_in = 'c' $
         else $
            imagetype_in = 'p'
         endif
   else: message, 'Must set no more than one of IMAGETYPE_IN, CONTOURS, PIXELS'
   endcase

imagetype = strupcase(imagetype_in)
case imagetype of
   'C': begin
        extend_cell = 0.0
        levels_open = !true
        end
   'P': begin
        extend_cell = 0.5
        levels_open = !false
        end
   else: message, 'Bad value for IMAGETYPE'
   endcase
c_overplot = !false ;%% Plot contour lines over filled contours (unimplemented)?

if not keyword_set(timenolabels) then timenolabels = !false

ntimes = n_elements(times)

sz = size(z)
case sz(0) of
   2: nchans = 1
   3: nchans = sz(3)
   endcase

if not keyword_set(ytitles) then $
   ytitles = strarr(nchans)
if not keyword_set(ztitles) then $
   ztitles = replicate(' ',nchans)
if not keyword_set(title) then $
   title = ''

; Make sure time axis is long enough to span outer pixels
timerange = ends(times, extend_cell=extend_cell)
if timerange(1) - timerange(0) le 6*3600 then $
   reftime = canmidnight(timerange(0)) $
else $
   reftime = canjan0(timerange(0))
plot_times = times - reftime
plot_timerange = timerange - reftime

xlen_legend = 0.02
xoffset_legend =  0.03
if not keyword_set(pos_regions) then $
   pos_regions = [0.1, 0.06, 0.9, 0.93]
pos_in_region = [0.0, 0.1, 1.0, 1.0]

!y.ticklen = -0.02	; Frequency axis ticks outward

; Choose levels for display

if keyword_set(levels_arr) then begin
   if not(keyword_set(nlevels_arr)) then $
      message, 'Must give both or neither of NLEVELS_ARR and LEVELS_ARR'
   endif $
else begin
   max_ncolors = 14	; About max number of contours or grey levels likely
   levels_arr = fltarr(max_ncolors, nchans)
   nlevels_arr = intarr(nchans)
   for chanx=0,nchans-1 do begin
      levels = enter_levels(z(*,*,chanx), ytitles(chanx), open=levels_open, $
         max_colors=max_ncolors)
      levels_arr(0,chanx) = levels
      nlevels_arr(chanx) = n_elements(levels)
      endfor
   endelse

!p.noerase = 1
region_multi, /init, [1,nchans], position=pos_regions, title=title

!x.range = plot_timerange
!x.style = 5
!p.title = ''

for chanx=0,nchans-1 do begin

   !y.title = ytitles(chanx)
   !y.range = ends(y, extend_cell=extend_cell)

   if chanx eq nchans-1 then begin
      xtitle = ''	; Use TIMESCALE default
      nolabels = timenolabels	;%% !false
      endif $
   else begin
      xtitle = ' '
      nolabels = !true
      endelse

   levels = levels_arr(0:nlevels_arr(chanx)-1, chanx)

   ; Choose colors. Needed if new LEVELS or new device.

   if levels_open then $
      ncolors = n_elements(levels) + 1 $
   else $
      ncolors = n_elements(levels) - 1

   colors = colors16(ncolors, backcolor=backcolor)
;%%   colors = $	;%% For several overplotted contours to each filled one
;%%      colors16([1,1,3,3,5,5,7,7,9,9,11,11,13,13], backcolor=backcolor)

   ; Set plot positions

   region_multi, chanx
   region_position, pos_in_region

   ; Set axes scales. Plot no axes or data.

   plot, [0,0], [0,0], /nodata, ystyle=5

   ; Plot image

   case imagetype of
      'P': tvxy, bin(z(*,*,chanx), levels, colors, backcolor), $
              plot_times, y
      'C': begin
           contour_prefill, z(*,*,chanx), plot_times, y, zz,xx,yy, $
              levels, colors, fill_levels, fill_colors
           contour, zz, xx, yy, levels=fill_levels, c_color=fill_colors, $
              ystyle=5, /overplot, /fill
;           if c_overplot then $		; Overplot contour lines
;              contour, zz, xx, yy, /noerase, /overplot, /follow, $
;                 levels=levels, ystyle=5, c_label=0
           end

      endcase

   ; Plot color legend

   color_legend, colors=colors, labels=strim_float(levels), $
      title=ztitles(chanx), open=levels_open, $
      position=[!x.window(1)+xoffset_legend, $
                !y.window(0), $
                !x.window(1)+xoffset_legend+xlen_legend, $
                !y.window(1)-2*y_ch_size(/normal)]

   ; Plot Y axes

   plot, [0], /nodata, $	; Plot y axes. No data.
      ystyle=1, ytitle=ytitles(chanx)

   ; Plot X axes

   !x.charsize=1.25	; Compensate CHARSIZE=0.8 in TIMESCALE
   timescale, plot_timerange(0), plot_timerange(1), title=xtitle, /top, $
      nolabels=nolabels, tic_factor=-nchans

   endfor ; chanx

region_multi, /reset
!p.noerase = 0
print, ' '
colors = colors16()	; To ensure some reasonable shades after program exit.

!p = psav
!x = xsav
!y = ysav
return

end
;+
; NAME:
;	CANTIME_PLOTS
; PURPOSE:
;	Stack plot of several time series against CANOPUS time.
; CALLING SEQUENCE:
;	cantime_plots, times, y [,ntimesvec]
; INPUTS:
;	TIMES = Vector or array of CANOPUS times.
;	   (Without midnight or Jan 0 subtracted)
;	   If 1D it is used for each row of Y.
;	   If 2D each row corresponds to one row of Y.
;	Y = 2D array of data to plot. Dimensions should be
;	   (N_ELEMENTS(TIMES), number of time series to plot).
;	   Second dimension may be 1.
; OPTIONAL INPUT PARAMETERS:
;	NTIMESVEC = vector of number of data points (t,y) in each time series.
;	   Number of elements equals number of time series.
;	   Ignored if TIMES is 1D.
;	   If absent then the whole of each row of TIMES is used.
; KEYWORD PARAMETERS:
;	POSITION = Vector of length 4 of coords of corners of whole data
;		window, relative to plot region. Defaults if absent.
;	TIMERANGE = Vector of length 2 containing start and end CANOPUS times
;		of the time axis.
;		Default is [earliest TIME value, latest TIME value].
;	YTITLES = string array of Y titles. Default is null strings.
;	TITLE = Main title for page. Default is null string.
;	YRANGES = 1D (2 elements) or 2D array (2,N) of yranges.
;		If 1D then same yrange is used for all channels.
;		Default is zeroes, i.e. auto.
;	YTICKS = scalar of vector of number of major tick intervals on each Y
;		axis. If scalar, it is used for all Y axes.
;		Mandatory if MARIA_STYLE is set (but could be zero for default).
;	YMINOR = scalar of vector of number of minor tick intervals between
;		major ticks on each Y axis. If scalar, it is used for all Y
;		axes.
;		Mandatory if MARIA_STYLE is set (but could be zero for default).
;	NOERASE: If present and non-zero, screen is not erased before plot.
;	TIMENOLABELS: If present and non-zero, the time axis is not labelled.
;	NODAYLABELS: If present and non-zero, dates in the time axis labels
;		are suppressed.
;	MARIA_STYLE: If present and nonzero, the timeseries are plotted in a
;		style similar to MARIA_STACK and IDL_BETAPLOT: All timeseries
;		at the same Y scale, vertical and horizontal dashed grid
;		lines, no Y axis labelling except for titles, a vertical bar
;		to indicate the Y scale.
;	UNIT_STRING = scalar string. Physical units to write on Y scale bar.
;		Used only if MARIA_STYLE is set. Default is null string.
;	MAX_VALUE = Scalar float. Any data value greater than MAX_VALUE is
;		taken as a missing data point. Default is 99999.0.
;	PSYMS = Vector integer, length equals number of time series.
;		Symbols which with to plot data. See !P.PSYM for values.
; SIDE EFFECTS:
;	Generates graphics.
; PROCEDURE:
;	Calls REGION_MULTI to establish subplots, TIMESCALE for X axes.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRC, 26 Jul 1991: Written in IDL2 by adapting
;	   CANTIMEPLOTS to handle time series of mixed lengths.
;	IFG 9-Oct-1991: Changed to new HARDCOPY call.
;	IFG 10-Oct-1991: Added POSITION keyword.
;	IFG 6-Nov-1991: Allowed: 1D TIMES, TIMESVEC absent, default TIMERANGE.
;		Moved NTIMESVEC from 1st to 3rd parameter.
;	IFG 6-Jul-1992: Allowed 1D YRANGES.
;	IFG 11 Dec 1992: Replaced TIMEAXIS call by TIMESCALE.
;	                 Added ON_ERROR.
;	IFG 22 Apr 1993: Added keywords MARIA_STYLE, UNIT_STRING.
;	   Introduced YRANGES_IN so YRANGES will not be changed by routine.
;	IFG 14 May 1993: Added keyword MAX_VALUE.
;	IFG 27 May 1992: Added keyword TIMENOLABELS.
;	IFG 18 Jun 1992: Added keyword PSYMS.
;       GRANT, 28 Sep 1993: Changed REGION_MULTI calls to suit new definition
;          of POSITION argument.
;	GRANT, 25 May 1994: Added keywords YTICKS, YMINOR.
;	GRANT, 25 Jul 1994: Added keyword NODAYLABELS.
;	GRANT, 18 Oct 1994: Eliminated conversion of 1D TIMES to long integer.
;-

pro cantime_plots, times_in, y, ntimesvec_in, $
   position=regions_pos, title=title, timerange=timerange, $
   ytitles=ytitles, yranges=yranges_in, yticks=yticks_in, yminor=yminor_in, $
   noerase=noerase, $
   maria_style=maria_style, unit_string=unit_string, max_value=max_value, $
   timenolabels=timenolabels, nodaylabels=nodaylab, psyms=psyms

on_error, 2
psav = !p
xsav = !x
ysav = !y

sy = size(y)
st = size(times_in)

; Set number of channels NCHANS

case sy(0) of
   1: nchans = 1
   2: nchans = sy(2)
   endcase

; Set vector NTIMESVEC. (To be used as 1D. Dimension might be 1.)

if (n_params() eq 2) or (st(0) eq 1) then $
   ntimesvec = replicate(st(1), nchans) $ ; NTIMESVEC_IN absent or TIMES_IN 1D
else $
   ntimesvec = ntimesvec_in		  ; NTIMESVEC_IN present and TIMES_IN 2D

; Set TIMES array. (To be used as 2D. Second dimension might be 1.)

if (st(0) eq 1) then begin	; TIMES_IN 1D
   ;%% times = lonarr(ntimesvec(0),nchans)
   times = make_array(ntimesvec(0), nchans, type=type(times_in))
   for chanx=0,nchans-1 do $
      times(0,chanx) = times_in
   endif $
else $				; TIMES_IN 2D
   times = times_in

if not keyword_set(timerange) then $
   ; Min of all first times to max of all last times
   timerange = [min(times(0,*)), $
      max(times( ntimesvec(indgen(nchans))-1 , indgen(nchans) ))]
if not keyword_set(regions_pos) then $
   regions_pos = [0.11, 0.07, 0.9, 0.94]
regions_pos_abs = subrect(/to_abs, pregion(), regions_pos)
if not keyword_set(title) then $
   title = ''
if not keyword_set(ytitles) then $
   ytitles = strarr(nchans)

;%% 940930 Need to arrange for suitable defaults for YRANGES, YTICKS, YMINOR
;%% if MARIA_STYLE set.
;%% Note that dims must be (2,NCHANS) even if /MARIA_STYLE.
;
;%% if keyword_set(maria_style) then $	; 940930: Introduced N_YAXIS_DESCRIP
;%%    n_yaxis_descrip = 1 $
;%% else $
;%%    n_yaxis_descrip = nchans

if keyword_set(yranges_in) then $
   yranges = yranges_in $
else $
   yranges = fltarr(2,nchans)
if keyword_set(maria_style) and (n_elements(yranges) ne 2) then $
   message, 'YRANGES must be a 2-element vector when MARIA_STYLE is set'
if (size(yranges)) (0) eq 1 then begin
   ; YRANGES is one dimensional
   yr1d = yranges
   yranges = fltarr(2,nchans)
   yranges(0,*) = yr1d(0)
   yranges(1,*) = yr1d(1)
   endif

if keyword_set(yticks_in) then $
   yticks = array(yticks_in) $
else $
   yticks = intarr(nchans)
if keyword_set(maria_style) and (n_elements(yticks) ne 1) then $
   message, 'YTICKS must have one element when MARIA_STYLE is set'
if n_elements(yticks) eq 1 then $
   ; YTICKS has one element
   yticks = replicate(yticks(0), nchans)

if keyword_set(yminor_in) then $
   yminor = array(yminor_in) $
else $
   yminor = intarr(nchans)
if keyword_set(maria_style) and (n_elements(yminor) ne 1) then $
   message, 'YMINOR must have one element when MARIA_STYLE is set'
if n_elements(yminor) eq 1 then $
   ; YMINOR has one element
   yminor = replicate(yminor(0), nchans)

if not keyword_set(unit_string) then unit_string = '' ; Used if /MARIA_STYLE
if not keyword_set(max_value) then max_value = 99999.0
if not keyword_set(timenolabels) then timenolabels = !false
if not keyword_set(psyms) then psyms = intarr(nchans)

if timerange(1) - timerange(0) le 6*3600 then $
   reftime = canmidnight(timerange(0)) $
else $
   reftime = canjan0(timerange(0))
plot_times = times - reftime
plot_timerange = timerange - reftime
pticklen = !p.ticklen

if (not keyword_set(noerase)) and (not !p.noerase) then $
   erase
!p.noerase = 1
   if keyword_set(maria_style) then begin

      ; MARIA style setup

      ; Set !P.POSITION to all regions, plot Y axes
      plot, /nodata, plot_timerange, [0,1], pos=regions_pos_abs, title=title, $
         xrange=plot_timerange, xstyle=5, $
         ystyle=1, yticklen=1e-6, ytickname=replicate(' ',10), ytitle=' '
      !x.ticklen = 0.0
      !x.charsize = 1.25	; Compensate CHARSIZE=0.8 in TIMESCALE
      timescale, plot_timerange(0), plot_timerange(1), title='', $
         nolabels=timenolabels, nodaylab=keyword_set(nodaylab), $
         /top, /vertical, tic_factor=-1	; With vertical grid lines
      !p.clip = !p.position

      ; Plot scale

      dx_norm_region =  regions_pos_abs(2)-regions_pos_abs(0)
      dy_norm_region = (regions_pos_abs(3)-regions_pos_abs(1)) / (nchans+1)
      x_norm_scale = regions_pos_abs(2) + 0.03*dx_norm_region	; Scalar
      y_norm_scale = regions_pos_abs(1) + [0,1]*dy_norm_region	; 2-vector
      plots, x_norm_scale*[1,1], y_norm_scale, /norm
      for i=0,1 do $
         plots, x_norm_scale+0.01*dx_norm_region*[-1,1], y_norm_scale(i), /norm
      xyouts, x_norm_scale+0.01*dx_norm_region, $
         mean(y_norm_scale)-0.5*pcharsize()*y_ch_size(/norm), /norm, $
         strim_float(yranges(1)-yranges(0)) + ' ' + unit_string

      ; Allow space (equal to half a subregion) at top and bottom before
      ; dividing into subregions

      region_multi, /init, [1,nchans], title='', $
         position=regions_pos + [0,1,0,-1]*0.5*dy_norm_region

   endif else begin

      ; Non-MARIA style setup

      region_multi, /init, [1,nchans], title=title, position=regions_pos
      endelse

   for chanx=0,nchans-1 do begin
      region_multi, chanx
      region_position, [0.0, 0.0, 1.0, 1.0]
      !p.ticklen = pticklen

      !x.range = plot_timerange
      !x.style = 5
      !y.title = ytitles(chanx)
      !y.range = yranges(*,chanx)
      !y.ticks = yticks(chanx)
      !y.minor = yminor(chanx)
      if keyword_set(maria_style) then !y.style = 5
      !p.title = ''

      plot, plot_times(0:ntimesvec(chanx)-1,chanx), $
         y(0:ntimesvec(chanx)-1,chanx), max_value=max_value, psym=psyms(chanx)
      if keyword_set(maria_style) then begin
         plot, plot_timerange, [0,0], linestyle=1	; Horizontal grid lines
         xyouts, $	;%% In X, "2*" before 0.04 is ad hoc for Helvetica
            plot_timerange(0) - 2*0.04*(plot_timerange(1)-plot_timerange(0)) - $
            strlen(ytitles(0))*pcharsize()*x_ch_size(/data), $
            -0.5*pcharsize()*y_ch_size(/data), ytitles(chanx), alignment=0.0
      endif else begin
         top = (chanx eq 0)
         nolabels = (chanx ne nchans-1) or timenolabels
         if nolabels then $
            xtitle = ' ' $
         else $
            xtitle = ''	; Use default
         !p.ticklen = pticklen*nchans
         !x.charsize = 1.25	; Compensate CHARSIZE=0.8 in TIMESCALE
         timescale, plot_timerange(0), plot_timerange(1), title=xtitle, $
            nolabels=nolabels, nodaylab=keyword_set(nodaylab), $
            top=top, tic_factor=-1
         endelse
      endfor ; chanx
   region_multi, /reset
!p.noerase = 0

!p = psav
!x = xsav
!y = ysav

return
end
