;+
; NAME:
;	DETREND
; PURPOSE:
;	Subtract a linear trend from a time series.
; CALLING SEQUENCE:
;	Result = DETREND(Y [,X] [,A,B])
; INPUTS:
;	Y	Vector of ordinate values.
; OPTIONAL INPUT PARAMETERS:
;	X	Vector of abcissa values. If omitted, then indices of Y are
;		used.
;	A	Y-intercept of trend, i.e. y(x=0).
;	B	Gradient of trend.
;	If A and B are not supplied, then they are determined by linear
;	regression.
; KEYWORD PARAMETERS:
;	MAX_VALUE: Points with Y ge MAX_VALUE are ignored for determining the
;	   trend and the corresponding points in Result are returned without
;	   the trend subtracted.
; OUTPUTS:
;	RESULT = Vector of ordinates with linear trend removed, i.e. Y-(A+B*X).
; PROCEDURE:
;	Uses LREGRESS.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 30 Oct 1990: Written in IDL1.
;	IFG  6 Feb 1991:  No changes needed for IDL2.
;	IFG 12 Apr 1991:  Added MISSING keyword.
;	IFG, 5 Jan 1993: Added ON_ERROR.
;	IFG, 3 Feb 1993: Renamed MISSING to MAX_VALUE.
;-

function detrend,y,x,a,b, max_value=max_value

on_error, 2
if n_params(0) eq 1 then x = findgen(n_elements(y))

; Remove missing values

xgood = x
ygood = y
if n_elements(max_value) gt 0 then begin
   bad  = where(y ge max_value, nbad)
   good = where(y lt max_value, ngood)
   if ngood eq 0 then message, 'All points in Y are flagged as missing'
   ; In case GE and LT are not complementary due to lack of precision
   if nbad+ngood ne n_elements(y) then $
      message, $
 'Numeric errors produce inconsistent number of missing and non-missing points'
   xgood = xgood(good)
   ygood = ygood(good)
   endif

; Detrend

if n_params(0) le 2 then lregress, xgood,ygood,a,b
result = y-(a+b*x)

; Replace missing points

if n_elements(max_value) gt 0 then $
   if nbad gt 0 then $
      result(bad) = y(bad)

return, result
end
