;+
; NAME:
;	IMSL_CCF
; PURPOSE:
;	Emulate the IMSL CCF subroutine for cross- and auto-correlations.
;	Parameters are identical to those of IMSL CCF.
; CALLING SEQUENCE:
;	IMSL_CCF,
;	   NOBS,X,Y,MAXLAG,IPRINT,ISEOPT,IMEAN,XMEAN,YMEAN,XVAR,CCV,CC,SECC
; INPUTS:
;	NOBS:	Number of observations in each time series. (Input)
;	X:	Vector of length NOBS containing the first time series. (Input)
;	Y:	Vector of length NOBS containing the second time series. (Input)
;	MAXLAG:	Maximum lag of cross-covariances and cross-correlations to be 
;		computed. (Input)
;	IPRINT:	Printing option. (Input)
;		IPRINT  Action
;		  0	No printing is performed.
;		  1	Print the means and variances.
;		  2	Print the means, variances and cross-covariances.
;		  3	Print the means, variances, cross-covariances,
;			cross-correlations, and standard errors of
;			cross-correlations.
;	ISEOPT:	Option for computing standard errors of cross-correlations. 
;		(Input)
;		ISEOPT  Action
;		  0	No standard errors are computed.
;		  1	Compute standard errors using Bartlett's formula.
;		  2	Compute standard errors using Bartlett's formula with
;			the assumption of no cross-correlation.
;	IMEAN:	Option for computing the mean. (Input)
;		IMEAN   Action
;		  0	XMEAN and YMEAN are user specified.
;		  1	XMEAN and YMEAN are set to the arithmetic means of X 
;			and Y.
;	XMEAN:	Estimate of the mean of time series X. (Input if IMEAN=0;
;		output if MEAN=1)
;	YMEAN:	Estimate of the mean of time series Y. (Input if IMEAN=0; 
;		output if MEAN=1)
; OUTPUTS:
;	XVAR:	Variance of X. (Output)
;	YVAR:	Variance of Y. (Output)
;	CCV:	Vector of length 2*MAXLAG+1 containing the cross-covariances 
;		between X and Y. (Output)
;		The cross-covariance at lag k corresponds to CCV(k+MAXLAG),
;		where k=-MAXLAG,...,-1,0,1,...,MAXLAG.
;	CC:	Vector of length 2*MAXLAG+1 containing the cross-correlations 
;		between X and Y. (Output)
;		The cross-correlation at lag k corresponds to CC(k+MAXLAG),
;		where k=-MAXLAG,...,-1,0,1,...,MAXLAG.
;	SECC:	Vector of length 2*MAXLAG+1 containing the standard errors
;		of the cross-correlations between X and Y. (Output)
;		The standard error of CC(k) is SECC(k),
;		where k=-MAXLAG,...,-1,0,1,...,MAXLAG.
; RESTRICTIONS:
;	No parameters are optional.
; PROCEDURE:
;	Note that the expression in the IMSL manual (STAT/LIBRARY Version 1.0,
;	April 1987, p.513) for var{^rhoxy(k)} is wrong. The expression used in
;	this program is taken from Box & Jenkins, "Time Series Analysis:
;	forecasting and control", 1976. The SECC output in the Gas Furnace
;	Data example in the IMSL manual is produced by the Box & Jenkins
;	formula, but not by the formula in the IMSL manual. 
;
;	Tested with the Gas Furnace Data example in the IMSL manual.
;	Argument verification code has not been tested.
;
;	In comments below, K = MAXLAG.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 27 July 1990.
;	IFG,  6 Feb 1991: Name changed from CCF to IMSL_CCF. IDL2.
;	IFG, 23 Jul 1991: Fixed mistake in comments describing CC argument.
;	IFG, 23 Dec 1992: Added ON_ERROR.
;-

pro ccf_varposk, nobs,maxlag,rhox,rhoy,rhoxy,varrhoxy

; Input	rhox(0:maxlag)
;	rhoy(0:maxlag)
;	rhoxy(0:2*maxlag)
;	nobs
;	maxlag
; Output varrhoxy (0:maxlag)

on_error, 2
px = fltarr(2*maxlag+1)		; IMSL indices = -K,...,K.
py = px
px(0) = rotate(rhox,2)
px(maxlag) = rhox
py(0) = rotate(rhoy,2)
py(maxlag) = rhoy
varrhoxy = fltarr(maxlag+1)

s1 = total(px*py)
;%% s5 = total(px) + total(px * px)/2 + total(py * py)/2        ; IMSL - wrong
s5 = total(rhoxy * rhoxy) + total(px * px)/2 + total(py * py)/2 ; Box & Jenkins
for k=0,maxlag do begin
   ;%% s2 = total(rhoxy(0:2*maxlag-2*k) * rhoxy(2*k:2*maxlag))  ; IMSL - wrong
   s2 = total(rhoxy(2*k:2*maxlag) * rotate(rhoxy(2*k:2*maxlag),2)) ; Box&Jenkins
   s3 = total(px(0:2*maxlag-k) * rhoxy(k:2*maxlag))
   s4 = total(rotate(rhoxy(k:2*maxlag),2) * py(k:2*maxlag))
   varrhoxy(k) = $
      (s1 + s2 - 2*rhoxy(k+maxlag)*(s3+s4) + (rhoxy(k+maxlag)^2)*s5) $
      / (nobs-k)
   endfor
return
end



pro imsl_ccf, nobs,x,y,maxlag,iprint,iseopt,imean,xmean,ymean,xvar,ccv,cc,secc

; For X and Y: IDL subscript = IMSL subscript - 1. i.e. IDL: 0,...,NOBS-1.

on_error, 2
if n_params() ne 13 then message, 'Must have exactly 13 arguments'
if nobs lt 2 then message, 'NOBS must be GE 2'
iprint = fix(iprint)
iseopt = fix(iseopt)
imean = fix(imean)
sx = size(x)
sy = size(y)
if (sx(0) ne 1) or (sy(0) ne 1) then message, 'X and Y must be vectors'
if (sx(1) ne nobs) or (sy(1) ne nobs) then message, $
   'X and Y must each have NOBS elements'
if (maxlag lt 1) or (maxlag ge nobs) then $
   message, 'Must have 1 <= MAXLAG < NOBS'
if (iprint lt 0) or (iprint gt 3) then message, 'Must have IPRINT = 0,1,2 or 3'
if (iseopt lt 0) or (iseopt gt 2) then message, 'Must have ISEOPT = 0,1 or 2'
if (imean lt 0) or (imean gt 1) then message, 'Must have IMEAN = 0, 1 or 2'

; Means

if imean eq 1 then begin
   xmean = float(total(x))/nobs	; FLOAT will force real all subsequent results
   ymean = float(total(y))/nobs
   endif

; Autocovariance and cross-covariance functions

sigmax = fltarr(maxlag+1)
sigmay = sigmax
sigmaxy = fltarr(2*maxlag+1)	; Subscripts: IMSL=-K,...,K, where K=MAXLAG
				; So, IDL subscript = IMSL subscript + MAXLAG
for k=0,maxlag do begin
   sigmax(k) = total( (x(0:nobs-k-1)-xmean) * (x(k:nobs-1)-xmean) )/nobs
   sigmay(k) = total( (y(0:nobs-k-1)-ymean) * (y(k:nobs-1)-ymean) )/nobs
   sigmaxy(k+maxlag) = total( (x(0:nobs-k-1)-xmean) * (y(k:nobs-1)-ymean) )/nobs
   endfor
for k=-maxlag,-1 do $
   sigmaxy(k+maxlag) = $
      total( (x(-k:nobs-1)-xmean) * (y(0:nobs+k-1)-ymean) )/nobs
xvar = sigmax(0)	; Variances
yvar = sigmay(0)
ccv = sigmaxy

; Autocorrelation and cross-correlation functions

rhox = sigmax/xvar
rhoy = sigmay/yvar
rhoxy = sigmaxy/sqrt(xvar*yvar)
cc = rhoxy

; Standard errors of cross-correlations

if iseopt eq 1 then begin   ; Bartlett's formula (general case)
   secc = fltarr(2*maxlag+1)
   ; Find secc(k), k = 0,...,K.
   ccf_varposk, nobs,maxlag,rhox,rhoy,rhoxy,varrhoxy
   secc(maxlag) = sqrt(varrhoxy)
   ; Find secc(k), k = -K,...,0. Repeat calculation as for k>0 but with X and Y
   ; swapped. Note that rhoyx(k) = rhoxy(-k), so use ROTATE(RHOXY,2) for rhoyx.
   ccf_varposk, nobs,maxlag,rhoy,rhox,rotate(rhoxy,2),varrhoxy
   secc(0) = sqrt(rotate(varrhoxy,2))
   endif ; iseopt = 1

if iseopt eq 2 then begin   ; Bartlett's formula assuming no cross-correlation
   s1 = 2*total(rhox * rhoy) - 1	; rhox(0)*rhoy(0) = 1
   secc = sqrt( s1/(nobs-abs(indgen(2*maxlag+1)-maxlag)) )
   endif ; iseopt = 2

; Printing

fmt = '(a,f15.6)'
ccfmt = '(i6,3f15.5)'

if iprint ne 0 then begin
   print, ' '
   print, 'Output from CCF'
   endif

if iprint ge 1 then begin
   print, ' '
   print, 'Mean of series X     = ',xmean, format=fmt
   print, 'Variance of series X = ',xvar , format=fmt
   print, ' '
   print, 'Mean of series Y     = ',ymean, format=fmt
   print, 'Variance of series Y = ',yvar , format=fmt
   endif

if iprint eq 2 then begin
   print, ' '
   print, '   Lag           CCV'
   print, ' '
   for k=-maxlag,maxlag do print, k, ccv(k+maxlag), format=ccfmt
   endif

if iprint eq 3 then begin
   print, ' '
   print, '   Lag           CCV             CC            SECC'
   print, ' '
   for k=-maxlag,maxlag do $
      print, k, ccv(k+maxlag), cc(k+maxlag), secc(k+maxlag), format=ccfmt
   endif
 
return
end
