;+
; NAME:
;	LIST_LIBRARY
; PURPOSE:
;	Extract first line of purpose documentation of one or more procedures.
; CATEGORY:
;	Help, documentation.
; CALLING SEQUENCE:
;	LIST_LIBRARY		;For prompting.
;	LIST_LIBRARY, Name 	;Extract documentation for procedure Name using
;				the current !PATH.
;
;	Example:
;		To produce a list of all of the modules in the IFG_MAR.TLB 
;		library, with a one-line description of each, use:
;		   LIST_LIBRARY, '*', PATH='@UTL$IDL$DAN:IFG_MAR.TLB', /BRIEF
; INPUTS:
;	Name:	A string containing the name of the procedure.
;		'*' to get all procedures in path.
; KEYWORDS:
;	BRIEF:	If set, each module generates only one line of output:
;		the name and first line of purpose, both extracted from the
;		file. The name is indicated by a line starting 	'; NAME:'. The
;		name text may either continue on the same line (after optional 
;		whitespace) or be on the second line as ';<whitespace><name>'.
;		Similarly for '; PURPOSE:'. The purpose information must follow
;		the name info.
;
;	FILE:	If this keyword is set, the output is sent to the file
;		"userlib.doc", in the current directory.
;
;	PRINT:	If set, this keyword sends the  output of LIST_LIBRARY to lpr.
;
;	PATH:	An optional directory/library search path.  This keyword uses
;		the same format	and semantics as !PATH.  If omitted, !PATH is
;		used.
; SIDE EFFECTS:
;	Output is produced on terminal or printer.
; RESTRICTIONS:
;	VMS only.
; PROCEDURE:
;	Straightforward.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 24 Jul 1992: Copied DOC_LIB_VMS to	LIST_LIBRARY,
;		added BRIEF keyword.
;	IFG, 20 Jan 1993: Recreated by inserting code to handle BRIEF into 
;	   DL_VMS (most recently modified 21 Sep 1992).
;-

pro list_library, NAME, FILE=file, PRINT=printflg, PATH = path, $
	NOFILEMSG=nofilemsg, BRIEF=brief

on_error,2			 ;Return to caller if an error occurs
params = n_params()
if n_elements(path) eq 0 then path = !path
do_print = keyword_set(printflg)
to_file = keyword_set(file)
do_filemsg = not keyword_set(nofilemsg)
if (to_file) then del_file = 0 else del_file = 1

if n_elements(name) eq 0 then begin	;Interactive query?
  name = ''
  printflg = 0
  read,'Name of procedure: ',name
endif

wild = name eq '*'		 ;True for all modules
;  name = strlowcase(name)	 ;make it always lower case
name = strupcase(name)	  ;make it always upper case

setlog,'_DOCUMENT',' ',table='LNM$JOB'  ;Logical name used for communic
cmd_file = !dir + '[bin]doc_library' ;Name & locn of cmd proc

;;cmd = '@'+ cmd_file + '  "[],'+path+'" '+name  ;To search curr directory
;;;; cmd = '@'+ cmd_file + '  "'+path+'" '+name  ;To omit current directory
;;spawn, cmd

; Convert path into a string array, one path section per array element
bpath = byte(path)
comma = (byte(','))(0)
loc = where(bpath eq comma, n_seg)
if (n_seg eq 0) then begin
  path_arr = path
endif else begin
  path_arr = strarr(n_seg + 1)
  last = 0
  for i = 0, n_seg-1 do begin
    path_arr(i) = string(bpath(last:loc(i)-1))
    last = loc(i)+1
  endfor
  path_arr(n_seg) = string(bpath(last:*))
endelse


; Define the logical IDL_DL_PATH to be a multi-valued logical containing
; the value of path_arr
for i = 0, n_elements(path_arr) - 1 do begin
	setlog, 'IDL_DL_PATH_' + strtrim(i, 2), path_arr(i)
endfor

; Spawn the command file to do the searching
spawn, '@'+ cmd_file + '  ' + name
for i = 0, n_elements(path_arr) - 1 do begin
	dellog, 'IDL_DL_PATH_' + strtrim(i, 2)
endfor
i = trnlog('_DOCUMENT', table='LNM$JOB', status)
if status eq ' ' then message, 'Error executing command file ' + cmd_file

if status eq "?" then message, 'Module ' + name + ' not found in library.'

i = strpos(status,' ')	  ;Found module, process
in_name = strmid(status, 0, i)  ;Input file name
to_delete = strmid(status, i+1,1) eq '1'  ;Delete input file flag
source = strmid(status, i+3, 100)	;Source file

if (to_file or do_print) then begin
  openw,outunit,'userlib.doc', /GET_LUN, PRINT=do_print, DELETE=del_file
  if (to_file and do_filemsg) then $
    message, 'Documentation is in file "userlib.doc".', /INFORMATIONAL
endif else begin
  openw,outunit,'sys$output', /GET_LUN, /MORE
endelse

if not wild then begin
	printf, outunit, '---- Module: ', STRUPCASE(name), ' ----'
	printf, outunit, '---- From:   ', source,'  -----'
	endif

openr,inunit,in_name, /GET_LUN, delete=to_delete ;Read text file, delete when done
A=''
WHILE NOT EOF(inunit) DO BEGIN
  READF,inunit,A
  IF STRMID(A,0,2) EQ ';+' THEN BEGIN
    READF,inunit,A
    if keyword_set(brief) then begin
      ; This section of code uses a "state variable" technique. Whenever a new 
      ; line of text is examined, BRIEF_STATUS identifies which of the 5
      ; sequential stages of the search process is in progress. The stages are 
      ; listed in the CASE statement comments.
      brief_status = 0
      name = ''
      purpose = ''
      WHILE STRMID(A,0,2) NE ';-' DO BEGIN
        case brief_status of
          0: begin ; Search for NAME header
            if strupcase(strmid(a,0,7)) eq '; NAME:' then begin
              name = strtrim(strmidend(a,7),2)
              brief_status = brief_status+1 	; Now look for name text
              endif
            if name ne '' then $
              brief_status = brief_status+1	; Header and name on same line
            end
          1: begin ; Search for name text
            name = strtrim(strmidend(a,1),2)	; Name text on separate line
            brief_status = brief_status+1
            end
          2: begin ; Search for PURPOSE header
            if strupcase(strmid(a,0,10)) eq '; PURPOSE:' then begin
              purpose = strtrim(strmidend(a,10),2)
              brief_status = brief_status+1 	; Now look for purpose text
              endif
            if purpose ne '' then $
              brief_status = brief_status+1	; Header and text on same line
            end
          3: begin ; Search for purpose text
            purpose = strtrim(strmidend(a,1),2) ; Purpose text on separate line
            brief_status = brief_status+1
            end
          4: begin ; Name and purpose search done
            dummy = 0
            end
          endcase
        readf, inunit, a
        ENDWHILE
      printf, outunit, $
        strmid(name+strof(' ',15),0,15) + ' ' + strmid(purpose,0,100)
      endif $ ; brief
    else $
      WHILE STRMID(A,0,2) NE ';-' DO BEGIN
           PRINTF,outunit,STRMID(A,1,100) & READF,inunit,A 
        ENDWHILE
    if wild and not keyword_set(brief) then printf, outunit, ""
    if not wild then goto, done
    ENDIF
ENDWHILE

done: FREE_LUN, inunit, outunit
END
