;+
; NAME:
;	UNWRAP_NOSHIFT
; PURPOSE:
;	Unwrap data vector without a final DC shift.
;	Remove data jumps due to the ends of a circular scale (e.g. 360 degs).
;	The first value is unchanged.
; CALLING SEQUENCE:
;	Result = UNWRAP_NOSHIFT( DATA_IN, MAXVAL)
; INPUTS:
;	DATA_IN = Vector of data which might contain jumps.
;	MAXVAL  = Max of range of DATA_IN values. Range is 0.0 to MAXVAL.
; OUTPUTS:
;	Result = Data vector with range possibly exceeding 0.0 to MAXVAL.
;	   Elements of Result differ from corresponding elements of DATA_IN by 
;	   integral multiples of MAXVAL. Result(0) equals DATA_IN(0).
; RESTRICTIONS:
;	Any increment in DATA_IN exceeding +/- 0.5*MAXVAL is treated as a jump 
;	caused by the value sequence crossing an end of the range 0.0 to MAXVAL.
; PROCEDURE:
;	The input vector is scanned from start to end.
;	At any increment outside +/- 0.5*MAXVAL, to bring the increment within
;	range the procedure increments or decrements the number of multiples of
;	MAXVAL needed to be applied to the remainder of the vector.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 16 July 1990
;	IFG, 6 May, 1991: IDL1? Looks like this will run under IDL2.
;	IFG, 5 Jan 1993: Renamed from DEDRUM_1D_NODC to UNWRAP_NOSHIFT.
;	   Added ON_ERROR.
;-

function unwrap_noshift, data_in,maxval

on_error, 2
data = data_in
half_maxval = maxval/2
nv = 0
datapr = data(0)

for i=0,n_elements(data)-1 do begin
   data(i) = data(i)+nv*maxval
   jump = data(i)-datapr
   if abs(jump) ge half_maxval then begin
      ; Shift between i-1 and i
      njump = round(abs(jump)/maxval)		; Number of cycles jumped
      sjump = sign(jump)			; Direction of jump
      nv = nv-sjump*njump  ; Shift remainder of series by this number of cycles
      data(i) = data(i)-sjump*njump*maxval	; Shift this point
      endif
   datapr = data(i)
   endfor

return, data
end
