;+
; NAME:
;	XYOUTS_MULTI
; PURPOSE:
;	Write multiple lines of text on a plot (more flexibly than XYOUTS).
; CALLING SEQUENCE:
;	XYOUTS_MULTI, X, Y, LINES
; INPUTS:
;	X:	Float scalar. X coordinate at which to put text.
;	Y:	Float scalar. Y coordinate at which to put text.
;	LINES:	String array. Each element is one line of text.
; KEYWORD PARAMETERS:
;	DATA:	If present and non-zero, X and Y are used as data coordinates.
;	DEVICE:	If present and non-zero, X and Y are used as device coords.
;	NORM:	If present and non-zero, X and Y are used as normalized coords.
;	Only one of DATA, DEVICE and NORM may be set. The default is /DATA.
;	LINE_SPACING: Float scalar. Vertical spacing between lines of text, in 
;	   units of the character height. Default is 1.
;	SIZE: Float scalar. Size of character, as for XYOUTS. Default is 1.
;	ALIGNMENT: Horizontal alignment of each line. Values as for XYOUTS.
;	VALIGNMENT: Vertical alignment of text.
;		0.0 to align at top of top line of text (default)
;		1.0 for bottom of bottom line
;		0.5 to center.
; RESTRICTIONS:
; PROCEDURE:
;	Convert coordinates if necessary, then call XYOUTS with vector for Y.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 29 June 1990.
;	IFG, 6 May 1991: Runs under IDL version 2. Not so necessary since '!c'
;	   was reintroduced in a recent release of IDL version 2.
;	IFG, UWO, 21 Apr 1993: Added VALIGNMENT keyword.
;	   Made default LINE_SPACING 1.
;-

pro xyouts_multi, x,y,lines, data=data, device=device, norm=norm, $
   line_spacing=line_spacing, size=size, alignment=alignment, $
   valignment=valignment

on_error, 2
if not keyword_set(line_spacing) then line_spacing = 1.0  ; Was 1.5
if n_elements(size) eq 0 then size = 1.0
if not keyword_set(alignment) then alignment = 0.0
if not keyword_set(valignment) then valignment = 0.0
dummy = where([keyword_set(data),keyword_set(device),keyword_set(norm)],count)
if count gt 1 then message, 'More than one of DATA, DEVICE and NORM set'
if count eq 0 then data = !true

size_lines = size(lines)
if (size_lines(0) ne 1) and (size_lines(size_lines(0)+1) ne 7) then $
   message, 'LINES must be a vector of strings'
nlines = n_elements(lines)

if keyword_set(data) then xy_dev = convert_coord( /data, /to_device, x, y)
if keyword_set(norm) then xy_dev = convert_coord( /norm, /to_device, x, y)
xyouts, xy_dev(0), xy_dev(1) - $
   line_spacing*size*!d.y_ch_size * (findgen(nlines)+1 - valignment*nlines), $
   lines, /device, size=size, alignment=alignment

return
end
