;+
; NAME:
;	ENTER_LEVELS
; PURPOSE:
;	Enter bin boundaries for color/greyscale scale.
;	Default automatic choice.
; CALLING SEQUENCE:
;	Result = ENTER_LEVELS( DATA, DATA_ID )
; INPUTS:
;	DATA = Array. It is expected that the entered levels will be chosen to 
;	   suit the range of values in DATA. DATA is used only as follows: the
;	   prompt printed includes the minimum and maximum of DATA as
;	   information; the minimum and maximum are used to select the default
;	   levels.
;	DATA_ID = Scalar string with which to label the data in the prompt.
; KEYWORD PARAMETERS:
;	MAX_VALUE: If present and nonzero then elements of DATA ge MAX_VALUE
;	   are ignored for determining the mininum and maximum.
;	MAX_COLORS = Maximum number of bins to be created between (and possibly 
;	   outside) the elements of Result. e.g. For a color scale (closed or
;	   open), MAX_COLORS should be the number of colors in the scale.
;	   Default is 14, implying maximum of 15 levels for closed, 13 for open.
;	CLOSED: If present and nonzero the outer boundaries of first and last
;	   bins are to be specified.
;	OPEN:   If present and nonzero the outer boundaries of first and last
;	   bins are to be unspecified, as for contouring.
;	POSNEG: If present and nonzero then default levels are centred on zero.
;	NOZERO: Ignored if POSNEG is not set. If NOZERO is present and nonzero
;	   then the zero value is omitted from the middle of the default levels.
; OUTPUTS:
;	Result = vector. Values correspond to expression entered in response to 
;	   prompt, or default if <RETURN>. Default is approximately 6 round
;	   values either just spanning (closed) or just spanned by (open) the 
;	   range of values in DATA.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 23 Oct 1991: Written in IDL2.
;	IFG, 5 Jan 1993: Added ON_ERROR.
;	IFG, 1 Apr 1993: Added keyword MAX_COLORS.
;	GRANT, 22 Sep 1993: Added MAX_NLEVELS argument to NICE_TICKS/LEVELS 
;	   call to return Result with length close to MAX_COLORS.
;	GRANT, 24 Sep 1993: Added keywords POSNEG, NOZERO.
;	   Corrected default to ignore missing data.
;-

function enter_levels, data_in, data_id, max_value=max_value, $
   max_colors=max_colors, open=open, closed=closed, posneg=posneg, nozero=nozero

on_error, 2
kc = keyword_set(closed)
ko = keyword_set(open)

; CASE !true OF ... doesn't work, since (NOT 0)  =  -1  <>  1 !
if ((not kc) and (not ko)) then closed = !true	; Default
if ((not kc) and      ko ) then closed = !false
if (     kc  and (not ko)) then closed = !true
if (     kc  and      ko ) then message, 'Conflicting keywords OPEN and CLOSED'

if not keyword_set(max_colors) then max_colors=14	; 14 colors on LJ-250

if closed then $
   max_nlevels = max_colors+1 $
else $
   max_nlevels = max_colors-1

data = data_in
if keyword_set(max_value) then $
   data = nonmissing(data, 0.9*max_value)
min_plot_data = min( data, max=max_plot_data )

print, 'Enter levels (vector, max ' + strim(max_nlevels) + $
   ' levels, default auto) for:'
levels = array(enter('FLOAT', '   ' + data_id + $
   ' (min=' + strim_float(min_plot_data) + $
   ', max=' + strim_float(max_plot_data) + ')', 1e21))
if n_elements(levels) gt max_nlevels then begin
   levels = levels(0:max_nlevels-1)
   message, /inform, 'List of levels truncated because too long'
   endif

if levels(0) gt 1e20 then begin
   if keyword_set(posneg) then begin
      levels = posneg_ticks(data, max_colors/2, nozero=keyword_set(nozero) )
      if not closed then $
         levels = levels(1:n_elements(levels)-2)
      endif $
   else begin ; posneg
      if closed then $
         levels = nice_ticks(data, max_nlevels) $
      else $
         levels = nice_levels(data, max_nlevels)
      endelse ; posneg
   endif ; levels(0)

return, levels(0 : (n_elements(levels) < max_nlevels) - 1 )
end
