;+
; NAME:
;	TVXY
; PURPOSE:
;	Plot a 2D array as an image of rectangles centred on { (X(i),Y(j)) }.
; CALLING SEQUENCE:
;	TVXY, Z, X, Y
; INPUTS:
;	Z = two-dimensional array of color indices of rectangles.
;		Dimensions must be ( N_ELEMENTS(X), N_ELEMENTS(Y) )
;	X = Vector of abscissae of rectangle centres.
;	Y = Vector of ordinates of rectangle centres.
; KEYWORDS:
;	DX = Scalar. Pixel size in X direction in data units.
;	   Used only if N_ELEMENTS(X) is 1. In that case, X does not indicate
;	   the pixel size to be used and so DX is used instead.
;	   For log X axis, DX is log10 of data ratio represented by pixel width.
;	DY = Scalar. Pixel size in Y direction in data units.
;	   Used only if N_ELEMENTS(Y) is 1. In that case, Y does not indicate
;	   the pixel size to be used and so DY is used instead.
;	   For log Y axis, DY is log10 of data ratio represented by pixel
;	   height.
; SIDE EFFECTS:
;	Graphics output.
; RESTRICTIONS:
;	Assumes values in X and Y are respectively arithmetic sequences (or
;	logarithmic for log axis). So
;	only first and last elements and the number of elements are significant
;	for each of X and Y.
;	N_ELEMENTS(Y) cannot be 1, since that would require a 2D array of
;	trailing dimension 1 be passed to TV. Could overcome this in a
;	later version by passing to TV arguments corresponding to
;	N_ELEMENTS(Y) EQ 2 and pixel height half of that desired.
;
;	Assumes Plot Data Window is set before entry.
; PROCEDURE:
;	Uses POLY2D to stretch image for devices with non-scalable pixels.
; MODIFICATION HISTORY:
;	Ian Grant, HIA/NRCC, 14 May 1991, IDL2.
;	IFG,  4 Jan 1993: Added ON_ERROR.
;	IFG, 15 Mar 1993: Added keyword DX to allow images with one column.
;	IFG, 16 Mar 1993: Modified to handle images with a side length
;	   comparable to or less than one device pixel. Now all images are
;	   forced to a size of at least one device unit in both directions.
;	GRANT, 27 Mar 1993: Changed to TV in data coordinates for scaleable
;	   pixels, because IDL bug fixed.
;	GRANT, 29 Oct 1993: Added keyword DY to allow images with one row.
;	   Easy now that IDL allows trailing dimensions of 1.
;	GRANT, 10 Nov 1993: Adapted for log axes, in addition to linear.
;-

pro tvxy, z, x, y, dx=dx, dy=dy
;stop
;print, 'dx = ',dx
;on_error, 2
sz = size(z)
nx = n_elements(x)
ny = n_elements(y)

if sz(0) ne 2 then message, 'Z does not have 2 dimensions'
if ((sz(1) ne nx) or (sz(2) ne ny)) then $
   message, 'Dimensions of Z are not ( N_ELEMENTS(X), N_ELEMENTS(Y) )'

xyrange_norm = convert_coord(ends(x), ends(y), /data, /to_norm)
;xyrange_norm(1,0) = xyrange_norm(1,0)/xyrange_norm(1,1)
;xyrange_norm(1,1)=1.000
xrange_norm = xyrange_norm(0,*)
yrange_norm = xyrange_norm(1,*)

; HALFDX is half pixel width in norm coords
if nx eq 1 then begin
   if not keyword_set(dx) then $
      message, 'If N_ELEMENTS(X) EQ 1 then keyword DX must be set'
   halfdx = 0.5*dx * !x.s(1)
   endif $
else $
   halfdx = 0.5 * (xrange_norm(1) - xrange_norm(0)) / (nx-1)

; HALFDY is half pixel height in norm coords
if ny eq 1 then begin
   if not keyword_set(dy) then $
      message, 'If N_ELEMENTS(Y) EQ 1 then keyword DY must be set'
   halfdy = 0.5*dy * !y.s(1)
   endif $
else $
   halfdy = 0.5 * (yrange_norm(1) - yrange_norm(0)) / (ny-1)

xrange_norm = xrange_norm + halfdx*[-1,1]
yrange_norm = yrange_norm + halfdy*[-1,1]

if !d.flags and 1 then $

   ; Device with scalable pixels (e.g. PostScript)

   tv, z, xrange_norm(0), yrange_norm(0), $
      xsize=xrange_norm(1)-xrange_norm(0), $
      ysize=yrange_norm(1)-yrange_norm(0), /norm $
else begin

   ; Device with non-scalable pixels

   ; Destination area in pixels
   ; Corners in pixels
   pdxy = convert_coord( xrange_norm, yrange_norm, /norm, /to_device)
   pdx = reform(pdxy(0,*))					; X limits
   pdy = reform(pdxy(1,*))					; Y limits
   sdx = pdx(1)-pdx(0)						; Size in X
   sdy = pdy(1)-pdy(0)						; Size in Y

   ; Expand input image to fit destination area and plot it

    tv, poly_2d(z, [[0,0],[float(nx)/sdx,0]], [[0,float(ny)/sdy],[0,0]], $
      0, (round(sdx) > 1), (round(sdy) > 1) ), pdx(0), pdy(0)
   endelse

return
end
